#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) 2007-2018 Etienne Robillard <tkadm30@yandex.com>
# All rights reserved.
"""Schevo database wrappers."""

__all__ = ['ConnectionError', 'DatabaseProxy']

#import schevo.database
from schevo.database import format_dbclass

# mulithreading support
import schevo
import schevo.mt

class ConnectionError(Exception):
    """Error connecting to the selected DB backend"""
    pass

class DatabaseProxy(object):
    """Creates and manages live ``Database`` objects using Proxy
    style attribute delegation.

    Usage::

        >>> from notmm.dbapi.orm import DatabaseProxy
        >>> db = DatabaseProxy('moviereviews') # access the "moviereviews" database
        >>> article = db.Article.findone(uid=2201)

    """

    db_version = 2
    cache = {}

    def __init__(self, db_name, db_connection_cls=None, db_debug_level=0,
        sync=True):

        self.DatabaseClass = format_dbclass[self.db_version]

        try:
            # Initialize the connection object
            self.conn = self.cache[db_name]
        except KeyError:
            try:
                self.conn = db_connection_cls(db_name)
                #self.conn = schevo.database.open(db_name)
                self.root = self.conn.get_root()
            except Exception as exc:
                raise ConnectionError(exc)
            else:
                self.cache[db_name] = self.conn
        else:
            if db_debug_level >= 1:
                # perform a quick sanity check
                assert 'SCHEVO' in self.root, 'Not a Schevo database or unexpected DB format: %r' % db_name

        setattr(self, 'db_name', str(db_name))

        self.initdb(self.conn, sync)

    def initdb(self, conn, sync=True, multithread=True):


        # Finalize db; setup proper multi-threading support
        db = self.DatabaseClass(conn)
        if sync:
            db._sync()
        if multithread:
            schevo.mt.install(db)
        setattr(self, '_db', db)

    def __getattr__(self, name):
        lock = self._db.read_lock()
        with lock:
            #lock.acquire()
            try:
                attr = getattr(self._db, name)
            except AttributeError:
                raise
        lock.release()
        return attr

    def __repr__(self):
        return "<Database: version=%d name=%s backend=%s>" % \
            (self.db_version, self.db_name, self.DatabaseClass)

    def commit(self):
        """Invoke underlaying ``commit`` method"""
        self._db._commit()
        return None



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys
import traceback
import logging
import hashlib

from notmm.utils.template import RequestContext
from notmm.utils.wsgilib.response import HTTPResponse
from notmm.utils.wsgilib.exc import (
    HTTPNotModifiedResponse,
    HTTPException
    )
from datetime import datetime
from time import ctime

from .utils import get_template_loader, TemplateException

log = logging.getLogger('notmm.utils.template')

TemplateLoader = get_template_loader()

__all__ = ['direct_to_template', 'render_template']

def render_template(template_name, ctx, charset='UTF-8', disable_unicode=False):
    t = TemplateLoader.get_template(template_name)
    if disable_unicode and t.output_encoding != charset:
        log.debug("Fixing output encoding to %s" % charset)
        setattr(t, 'output_encoding', charset)
    try:
        if charset == 'UTF-8' and not disable_unicode:
            value = t.render_unicode(data=ctx)
        else:
            value = t.render(data=ctx)
    except TemplateException:
        # Template error processing a unicode template
        # with Mako
        exc_type, exc_value, exc_tb = sys.exc_info()
        rawtb = ''.join([item for item in traceback.format_tb(exc_tb)])
        raise TemplateException(content=rawtb)
    return value

def direct_to_template(request, template_name, extra_context={},
    status=200, charset='UTF-8', mimetype='text/html',
    disable_unicode=False):
    """
    Generic view for returning a Mako template inside a simple
    ``HTTPResponse`` instance.

    """
    # Make sure ctx has our stuff
    if not isinstance(extra_context, RequestContext):
        ctx = RequestContext(request, extra_context)
    else:
        ctx = extra_context

    response = render_template(template_name, ctx)
    h = hashlib.md5(response.encode(charset))

    httpheaders = (
        ('Last-Modified', ctime()),
        ('If-None-Match', h.hexdigest())
    )

    return HTTPResponse(response, request=request,
        headers=httpheaders, status=status, mimetype=mimetype)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import unittest
import demjson
import test_response
from notmm.utils.wsgilib import HTTPResponse as BaseHTTPResponse

ResponseClass = BaseHTTPResponse

class JSONResponseTestCase(test_response.HTTPResponseTestCase):
    
    mimetype = 'application/json'
    charset = 'utf-8'
    
    def setUp(self):
        self.wsgi_app = ResponseClass(
            demjson.encode({'key':'value'}),
            mimetype=self.mimetype,
            charset=self.charset)
    
    def test_write(self):
        pass




from django.conf.urls.defaults import *
from livesettings import config_value, config_get_group

config = config_get_group('PAYMENT_AUTOSUCCESS')

urlpatterns = patterns('',
     (r'^$', 'payment.modules.autosuccess.views.one_step', {'SSL':False}, 'AUTOSUCCESS_satchmo_checkout-step2'),
     (r'^success/$', 'payment.views.checkout.success', {'SSL':False}, 'AUTOSUCCESS_satchmo_checkout-success'),
)



# Copyright (C) 2004,2005 by SICEm S.L.
# Copyright (C) 2005 Red Hat, Inc.
# Copyright (C) 2006 Async Open Source
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gobject
import pango
import gtk
from gtk import gdk

class WorkSpaceChild(object):
    """Attributes of a child widget stored in WorkSpace"""

    DECORATION_PAD = 5

    def __init__(self, widget):
        self.layout = None
        self.widget = widget
        self.x = 40
        self.y = 40
        self.window_x = 0
        self.window_y = 0
        self.title_height = 0
        self.decoration_x = 0
        self.decoration_y = 0
        self.decoration_width = 1
        self.decoration_height = 1

    def set_parent(self, parent):
        self.layout = parent.create_pango_layout('')
        self.layout.set_ellipsize(pango.ELLIPSIZE_END)
        self.layout.set_alignment(pango.ALIGN_LEFT)
        self.widget.set_parent(parent)

    def set_title(self, title):
        self.layout.set_text(title)
        extents = self.layout.get_pixel_extents()
        self.title_height = extents[1][3]

    def compute_size(self):
        parent = self.widget.get_parent()
        self.window_x = self.x + parent.allocation.x + parent.border_width
        self.window_y = self.y + parent.allocation.y + parent.border_width
        self.decoration_x = self.window_x - self.DECORATION_PAD
        self.decoration_y = (self.window_y -
                             self.DECORATION_PAD*2 - self.title_height)
        w, h = self.widget.size_request()
        self.decoration_width = w + self.DECORATION_PAD*2
        self.decoration_height = (h + self.DECORATION_PAD*2 +
                                  self.title_height + self.DECORATION_PAD)
        self.layout.set_width(w * pango.SCALE)

    def window_point_in_decoration(self, x, y):
        if y < self.decoration_y:
            return False
        elif y > self.decoration_y + self.decoration_height:
            return False
        elif x < self.decoration_x:
            return False
        elif x > self.decoration_x + self.decoration_width:
            return False
        elif y < self.window_y:
            return True
        elif x < self.decoration_x + self.DECORATION_PAD:
            return True
        elif x > (self.decoration_x +
                  self.decoration_width - self.DECORATION_PAD):
            return True
        elif y > (self.decoration_y +
                  self.decoration_height - self.DECORATION_PAD):
            return True
        else:
            return False

    def window_point_resize(self, x, y):
        if (x > (self.decoration_x +
                 self.decoration_width -
                 self.DECORATION_PAD) and
            y > (self.decoration_y +
                 self.decoration_height -
                 self.DECORATION_PAD)):
            return True
        return False

def set_adjustment_upper(adj, upper, always_emit):
    changed = False
    value_changed = False

    min = max(0.0, upper - adj.page_size)

    if upper != adj.upper:
        adj.upper = upper
        changed = True

    if adj.value > min:
        adj.value = min
        value_changed = True

    if changed or always_emit:
        adj.changed()
    if value_changed:
        adj.value_changed()

def new_adj():
    return gtk.Adjustment(0.0, 0.0, 0.0,
                          0.0, 0.0, 0.0)

(CHILD_NONE,
 CHILD_MOVE,
 CHILD_RESIZE) = list(range(3))

class WorkSpace(gtk.Container):
    __gsignals__ = dict(set_scroll_adjustments=
                        (gobject.SIGNAL_RUN_LAST, None,
                         (gtk.Adjustment, gtk.Adjustment)))
    def __init__(self):
        self._children = []
        self._moving_child = None
        self._moving_start_x_pointer = 0
        self._moving_start_y_pointer = 0
        self._moving_start_x_position = 0
        self._moving_start_y_position = 0
        self._action_type = CHILD_NONE
        self._width = 100
        self._height = 100
        self._hadj = None
        self._vadj = None
        self._vadj_changed_id = -1
        self._hadj_changed_id = -1
        self._bin_window = None

        gtk.Container.__init__(self)
        self.add_events(gdk.BUTTON_PRESS_MASK)

        if not self._hadj or not self._vadj:
            self._set_adjustments(self._vadj or new_adj(),
                                  self._hadj or new_adj())

    # Public API
    def set_size(self, width, height):
        if self._width != width:
            self._width = width
        if self._height != height:
            self._height = height
        if self._hadj:
            set_adjustment_upper(self._hadj, self._width, False)
        if self._vadj:
            set_adjustment_upper(self._vadj, self._height, False)

        if self.flags() & gtk.REALIZED:
            self._bin_window.resize(max(width, self.allocation.width),
                                    max(height, self.allocation.height))

    def set_widget_title(self, widget, title):
        """
        @param widget:
        @param title:
        """
        child = self._get_child_from_widget(widget)
        child.set_title(title)
        if child.widget.flags() & gtk.VISIBLE:
            self.queue_resize()

    def set_child_position(self, widget, x, y):
        """
        @param widget:
        @param x:
        @param y:
        """
        child = self._get_child_from_widget(widget)
        self._set_child_position(child, x, y)

    # Child callbacks
    def _on_child_realize(self, window):
        window.window = gdk.Window(
            window.get_parent_window(),
            window_type=gdk.WINDOW_CHILD,
            x=window.allocation.x,
            y=window.allocation.y,
            width=window.allocation.width,
            height=window.allocation.height,
            wclass=gdk.INPUT_OUTPUT,
            visual=window.get_visual(),
            colormap=window.get_colormap(),
            event_mask=(window.get_events() |
                        (gdk.EXPOSURE_MASK |
                         gdk.BUTTON_PRESS_MASK |
                         gdk.BUTTON_RELEASE_MASK |
                         gdk.KEY_PRESS_MASK |
                         gdk.KEY_RELEASE_MASK |
                         gdk.ENTER_NOTIFY_MASK |
                         gdk.LEAVE_NOTIFY_MASK |
                         gdk.STRUCTURE_MASK)))
        window.window.set_user_data(self)
        window.style.attach(window.window)

        window.window.enable_synchronized_configure()

    def _on_child_size_allocate(self, window, allocation):
        window.allocation = allocation
        if window.flags() & gtk.REALIZED:
            window.window.move_resize(*allocation)

        if window.child and window.child.flags() & gtk.VISIBLE:
            child_allocation = (
                window.border_width,
                window.border_width,
                max(1, window.allocation.width - window.border_width * 2),
                max(1, window.allocation.height - window.border_width * 2))
            window.child.size_allocate(child_allocation)

    # Private

    def _set_child_position(self, child, x, y, resize=True):
        if child.x != x or child.y != y:
            child.x, child.y = x, y

            if resize and child.widget.flags() & gtk.VISIBLE:
                self.queue_resize()

    def _get_child_from_widget(self, widget):
        for child in self._children:
            if child.widget == widget:
                return child
        else:
            raise AssertionError

    def _pick_child(self, window_x, window_y):
        reversed = list(self._children)
        reversed.reverse()
        for c in reversed:
            if c.window_point_in_decoration(window_x, window_y):
                return c
        return None

    def _begin_move_child(self, child, x, y, time):
        if self._moving_child != None:
            raise AssertionError("can't move two children at once")

        mask = (gdk.BUTTON_RELEASE_MASK | gdk.BUTTON_RELEASE_MASK |
                gdk.POINTER_MOTION_MASK)
        grab = gdk.pointer_grab(self.window, False, mask, None, None,
                                int(time))
        if grab != gdk.GRAB_SUCCESS:
            raise AssertionError("grab failed")

        self._children.remove(child)
        self._children.append(child)

        self._moving_child = child
        self._moving_start_x_pointer = x
        self._moving_start_y_pointer = y
        self._moving_start_x_position = child.x
        self._moving_start_y_position = child.y
        w, h = child.widget.get_size_request()
        self._moving_start_w, self._moving_start_h = w, h

    def _update_move_child (self, x, y):
        child = self._moving_child
        if not child:
            return

        dx = x - self._moving_start_x_pointer
        dy = y - self._moving_start_y_pointer
        if self._action_type == CHILD_MOVE:
            self._set_child_position(child,
                                     self._moving_start_x_position + dx,
                                     self._moving_start_y_position + dy)

        if self._action_type == CHILD_RESIZE:
            child.widget.set_size_request(self._moving_start_w + dx,
                                          self._moving_start_h + dy)

            self.queue_draw()

    def _end_move_child(self, time):
        if not self._moving_child:
            return

        gdk.pointer_ungrab(int(time))
        self._moving_child = None

    def _set_adjustments(self, hadj, vadj):
        if not hadj and self._hadj:
            hadj = new_adj()

        if not vadj and self._vadj:
            vadj = new_adj()

        if self._hadj and self._hadj != hadj:
            self._hadj.disconnect(self._hadj_changed_id)

        if self._vadj and self._vadj != vadj:
            self._vadj.disconnect(self._vadj_changed_id)

        need_adjust = False

        if self._hadj != hadj:
            self._hadj = hadj
            set_adjustment_upper(hadj, self._width, False)
            self._hadj_changed_id = hadj.connect(
                "value-changed",
                self._adjustment_changed)
            need_adjust = True

        if self._vadj != vadj:
            self._vadj = vadj
            set_adjustment_upper(vadj, self._height, False)
            self._vadj_changed_id = vadj.connect(
                "value-changed",
                self._adjustment_changed)
            need_adjust = True

        if need_adjust and vadj and hadj:
            self._adjustment_changed()

    def _adjustment_changed(self, adj=None):
        if self.flags() & gtk.REALIZED:
            self._bin_window.move(int(-self._hadj.value),
                                  int(-self._vadj.value))
            self._bin_window.process_updates(True)

    # GtkWidget

    def do_realize(self):
        self.set_flags(self.flags() | gtk.REALIZED)
        self.window = gdk.Window(self.get_parent_window(),
                                 width=self.allocation.width,
                                 height=self.allocation.height,
                                 window_type=gdk.WINDOW_CHILD,
                                 wclass=gdk.INPUT_OUTPUT,
                                 event_mask=(self.get_events() |
                                             gdk.EXPOSURE_MASK))
        self.window.set_user_data(self)
        self.window.move_resize(*self.allocation)

        self._bin_window = gdk.Window(
            self.window,
            window_type=gdk.WINDOW_CHILD,
            x=int(-self._hadj.value),
            y=int(-self._vadj.value),
            width=max(self._width, self.allocation.width),
            height=max(self._height, self.allocation.height),
            colormap=self.get_colormap(),
            wclass=gdk.INPUT_OUTPUT,
            event_mask=(self.get_events() | gdk.EXPOSURE_MASK |
                        gdk.SCROLL_MASK))
        self.style.set_background(self._bin_window, gtk.STATE_NORMAL)
        self._bin_window.set_user_data(self)

        self.set_style(self.style.attach(self.window))

        for child in self._children:
            child.widget.set_parent_window(self._bin_window)

    def do_unrealize(self):
        self._bin_window.set_user_data(None)
        self._bin_window = None
        self.window.set_user_data(None)

    def do_expose_event(self, event):
        window = event.window

        gc = gdk.GC(window)
        gc.set_rgb_fg_color(self.style.text[self.state])

        for c in self._children:
            window.draw_rectangle(self.style.base_gc[self.state], True,
                                  c.decoration_x, c.decoration_y,
                                  c.decoration_width, c.decoration_height)
            window.draw_rectangle(gc, False,
                                  c.decoration_x, c.decoration_y,
                                  c.decoration_width, c.decoration_height)

            window.draw_line(gc,
                             c.decoration_x,
                             c.window_y - c.DECORATION_PAD,
                             c.decoration_x + c.decoration_width,
                             c.window_y - c.DECORATION_PAD)

            window.draw_layout(gc,
                               c.decoration_x + c.DECORATION_PAD,
                               c.decoration_y + c.DECORATION_PAD - 2,
                               c.layout)

            self.propagate_expose(c.widget, event)

    def do_map(self):
        self.set_flags(self.flags() | gtk.MAPPED)
        for child in self._children:
            flags = child.widget.flags()
            if flags & gtk.VISIBLE:
                if not (flags & gtk.MAPPED):
                    child.widget.map()
        self._bin_window.show()
        self.window.show()

    def do_size_request(self, req):
        req.width = 0
        req.height = 0

        for child in self._children:
            if child.widget.flags() & gtk.VISIBLE:
                child.compute_size()
                req.width = max(req.width,
                                child.decoration_x + child.decoration_width)
                req.height = max(req.height,
                                 child.decoration_y + child.decoration_height)
            child.widget.size_request()

        req.width = req.width + self.border_width * 2
        req.height = req.height + self.border_width * 2

    def do_size_allocate(self, allocation):
        self.allocation = allocation
        for c in self._children:
            if c.widget.flags() & gtk.VISIBLE:
                child_req = c.widget.get_child_requisition()
                w, h = child_req
                c.widget.size_allocate((c.window_x, c.window_y, w, h))

        if self.flags() & gtk.REALIZED:
            self.window.move_resize(*allocation)
            self._bin_window.resize(max(self._width, allocation.width),
                                    max(self._height, allocation.height))

        self._hadj.page_size = allocation.width
        self._hadj.page_increment = allocation.width * 0.9
        self._hadj.lower = 0
        set_adjustment_upper(self._hadj,
                             max(allocation.width, self._width), True)

        self._vadj.page_size = allocation.height
        self._vadj.page_increment = allocation.height * 0.9
        self._vadj.lower = 0
        self._vadj.upper = max(allocation.height, self._height)
        set_adjustment_upper(self._vadj,
                             max(allocation.height, self._height), True)

    def do_set_scroll_adjustments(self, hadj, vadj):
        self._set_adjustments(hadj, vadj)

    def do_key_press_event(self, event):
        if event.keyval == gtk.keysyms.Escape:
            self._end_move_child(event.time)

    def do_button_press_event(self, event):
        if self._moving_child != None:
            return

        x, y = int(event.x), int(event.y)

        child = self._pick_child(x, y)
        if child == None:
            return

        if child.window_point_resize(x, y):
            self._action_type = CHILD_RESIZE
        else:
            self._action_type = CHILD_MOVE

        self._begin_move_child(child, x, y, event.time)

    def do_button_release_event(self, event):
        self._update_move_child(int(event.x), int(event.y))
        self._end_move_child(event.time)

    def do_motion_notify_event(self, event):
        if self._moving_child != None:
            self._update_move_child(int(event.x), int(event.y))

    # GtkContainer

    def do_forall(self, internals, callback, data):
        for child in self._children:
            callback(child.widget, data)

    def do_add(self, widget):
        child = WorkSpaceChild(widget)
        self._children.append(child)
        if self.flags() & gtk.REALIZED:
            widget.set_parent_window(self._bin_window)

        widget.connect('realize', self._on_child_realize)
        widget.connect('size-allocate', self._on_child_size_allocate)

        if isinstance(widget, gtk.Window):
            widget.unset_flags(gtk.TOPLEVEL)
            widget.set_resize_mode(gtk.RESIZE_PARENT)
            widget.set_size_request(300, 200)

        child.set_parent(self)

        if widget.flags() & gtk.VISIBLE:
            self.queue_resize()
        self.queue_draw()

    def do_remove(self, widget):
        child = self._get_child_from_widget(widget)

        if self._moving_child == child:
            self._end_move_child(0)

        was_visible = widget.flags() & gtk.VISIBLE
        self._children.remove(child)
        widget.unparent()

        if was_visible:
            self.queue_resize()
        self.queue_draw()

if gtk.pygtk_version >= (2, 8):
    WorkSpace.set_set_scroll_adjustments_signal('set-scroll-adjustments')

gobject.type_register(WorkSpace)

class EmbeddableMessageDialog(gtk.MessageDialog):
    """Message dialog with crazy hacks so you can embed it as a non-toplevel"""

    ### FIXME this class has to be generalized to all kinds of toplevel window,
    ### which should be trivial
    ### A better approach might be to do this entirely via signal
    ### connection rather
    ### than via subclass. Then you could have a make_embeddable() function
    ### that took any toplevel.

    __gsignals__ = {
        'realize': 'override',
        'size-allocate' : 'override',
        'show' : 'override',
        'hide' : 'override',
        'map' : 'override',
        'unmap' : 'override'
        }

    def __init__(self):
        #gtk.MessageDialog.__init__(self,
        #                           type=gtk.MESSAGE_INFO,
        #                           buttons=gtk.BUTTONS_OK)
        constructor = gobject.GObject.__init__
        constructor(self, message_type=gtk.MESSAGE_INFO,
                    buttons=gtk.BUTTONS_OK)
        self.label.set_text('Hello')

        self.unset_flags(gtk.TOPLEVEL)

    def do_realize(self):
        self.set_flags(self.flags() | gtk.REALIZED)

        self.window = gdk.Window(
            self.get_parent_window(),
            window_type=gdk.WINDOW_CHILD,
            x=self.allocation.x,
            y=self.allocation.y,
            width=self.allocation.width,
            height=self.allocation.height,
            wclass=gdk.INPUT_OUTPUT,
            visual=self.get_visual(),
            colormap=self.get_colormap(),
            event_mask=(self.get_events() | gdk.EXPOSURE_MASK |
                        gdk.KEY_PRESS_MASK | gdk.KEY_RELEASE_MASK |
                        gdk.ENTER_NOTIFY_MASK | gdk.LEAVE_NOTIFY_MASK |
                        gdk.STRUCTURE_MASK))

        self.window.set_user_data(self)
        self.style.attach(self.window)
        self.style.set_background(self.window, gtk.STATE_NORMAL)

        self.window.enable_synchronized_configure()

    def do_size_allocate(self, allocation):
        self.allocation = allocation
        if self.flags() & gtk.REALIZED:
            self.window.move_resize(*allocation)

        if self.child and self.child.flags() & gtk.VISIBLE:
            child_allocation = (
                self.border_width,
                self.border_width,
                max(1, self.allocation.width - self.border_width * 2),
                max(1, self.allocation.height - self.border_width * 2))
            self.child.size_allocate (child_allocation)

    def do_show(self):
        gtk.Bin.do_show(self)

    def do_hide(self):
        gtk.Bin.do_hide(self)

    def do_map(self):
        gtk.Bin.do_map(self)

    def do_unmap(self):
        gtk.Bin.do_unmap(self)

def main():
    window = gtk.Window()
    window.connect('delete-event', gtk.main_quit)
    window.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("white"))
    window.set_size_request(500, 500)
    sw = gtk.ScrolledWindow()
    sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
    window.add(sw)

    table = WorkSpace()
    table.set_border_width(20)

    sw.add_with_viewport(table)

    dialog = EmbeddableMessageDialog()

    table.add(dialog)

    button = gtk.Button("A Button")
    table.add(button)

    table.show_all()
    req = table.size_request()
    window.set_default_size(req[0] + 10, req[1] + 10)
    window.show_all()

    gtk.main()

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright 2017-2019 Etienne Robillard <tkadm30@yandex.com>
# See NOTICE file for details.
import logging
log = logging.getLogger(__name__)
from notmm.controllers.base import BaseController, sessionmanager
from notmm.controllers.wsgi import WSGIController
#from wsgi_oauth2.client import google_client

__all__ = ['GoogleController',]

class GoogleController(BaseController):
    scope = 'email'

    #oauth_client_class = google_client
    oauth_client_version = "2.0"
    oauth_client_debug = True

    def __init__(self, client, secret, **kwargs):
        super(GoogleController, self).__init__()
        self._client = client
        self._secret = secret
        self._login_path = kwargs['login_path']
        if self.oauth_client_debug:
            log.debug(self._client)   
    
    def application(self, environ, start_response):
        with sessionmanager(environ):
            req = self.request_class(environ)
            #self.init_request(req)
            app = self._client.wsgi_middleware(self, secret=self._secret, 
                login_path=self._login_path)
        return app(environ, start_response)




# Copyright (C) 2004,2005 by SICEm S.L.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gtk
import gobject
from kiwi.utils import gsignal, type_register

class GAction(object):
    """A GAction has the same information that a GtkAction but it
    is easier to work with. The menubar/toolbar editor code will
    convert this GActions to GtkActions and viceversa.
    """
    def __init__(self, parent, name='', label='',
                 short_label='', is_important=False,
                 tooltip='', stock_id=None, callback='', accel=''):
        # parent GActionGroup
        self.parent = parent

        self.name = name
        self.label = label
        if short_label:
            self.short_label = short_label
        else:
            self.short_label = label
        self.is_important = is_important
        self.tooltip = tooltip
        self.stock_id = stock_id
        self.callback = callback
        self.accelerator = accel

    #@classmethod
    def new(cls, gtk_action, parent):
        """Create a new GAction from a GtkAction and a GtkActionGroup"""
        # This code is similar to code in the loader, investigate
        # if we can use more code reusage
        name = gtk_action.get_name()
        label = gtk_action.get_property('label')
        short_label = gtk_action.get_property('short-label')
        is_important = gtk_action.get_property('is-important')
        tooltip = gtk_action.get_property('tooltip')
        stock_id = gtk_action.get_property('stock-id') or None
        gaction = cls(parent, name, label, short_label, is_important,
                      tooltip, stock_id)

        # check if it has accelerator
        accel_entry = gtk.accel_map_lookup_entry('<Actions>/%s/%s' %
                                                 (parent.name, name))
        if accel_entry:
            key, modifier = accel_entry
            if key != 0:
                gaction.accelerator = gtk.accelerator_name(key, modifier)

        # check if it has signal handler
        callback = gtk_action.get_data('handler')
        if callback:
            gaction.callback = callback

        return gaction
    new = classmethod(new)

    def __str__(self):
        return "%s %s %s %u %s %s %s" % (
            self.name, self.label, self.short_label, self.is_important,
            self.stock_id, self.accelerator, self.tooltip)

    def get_name(self):
        return self.name

class GActionGroup(gobject.GObject):
    """A GActionGroup is just a list of GActions with a name.
    Associated with each GActionGroup there is a GtkActionGroup with
    is keep synchronized with the GActionGroup:

        - As we can't change the name of a GtkActionGroup, if the
        user change the name of a GActionGroup we have to regenerate
        the GtkActionGroup again, copying the actions from the old one
        to the new one.

        - When an action is added or removed, we create or destroy the
        GtkAction associated with it. If an action is only changed, we
        regenerate the GtkAction and tell the UIManager to update itself
        by removing the GtkActionGroup and then addind it again.
        Anyone knows a better way to do so?
    """
    gsignal('add-action', object)
    gsignal('remove-action', object)

    def __init__(self, name=''):
        gobject.GObject.__init__(self)
        self._name = name
        self._actions = {}
        self._uimanager = None

    #@classmethod
    def new(cls, action_group):
        return cls(action_group.get_name())
    new = classmethod(new)

    def create_gtk_action_group(self, uimanager):
        self._uimanager = uimanager
        gtk_action_group = gtk.ActionGroup(self.name)
        gtk_action_group.set_data('gaction_group', self)
        self.set_data('gtk_action_group', gtk_action_group)
        self._uimanager.insert_action_group(gtk_action_group, 0)
        return gtk_action_group

    def destroy_gtk_action_group(self):
        gtk_action_group = self.get_data('gtk_action_group')
        self.set_data('gtk_action_group', None)
        self._uimanager.remove_action_group(gtk_action_group)

    def get_name(self):
        return self._name

    def set_name(self, new_name):
        """GtkAction groups can't change their name

        That's why we need to copy the old one into a new one
        """
        self._name = new_name

        if self._uimanager is None:
            return

        gtk_action_group = self.get_data('gtk_action_group')
        self._uimanager.remove_action_group(gtk_action_group)
        new_gtk_action_group = self.create_gtk_action_group(self._uimanager)

        for action in gtk_action_group.list_actions():
            gtk_action_group.remove_action(action)
            new_gtk_action_group.add_action(action)

    name = property(get_name, set_name)

    def add_action(self, action):
        self._actions[action.get_name()] = action

        self._add_gtk_action(action)

        self.emit('add-action', action)

    def add_actions(self, actions):
        # now create the default actions
        for (name, label, short, is_important,
             tooltip, stock_id, callback, acc) in actions:
            # check if the action already exists
            if name not in self._actions:
                ga = GAction(self, name, label, short, is_important,
                             tooltip, stock_id, callback, acc)
                self.add_action(ga)
            # XXX maybe we still want to add the action if its contents
            # are not exactly the same as the old one

    def _add_gtk_action(self, action):
        gtk_action = gtk.Action(action.name, action.label,
                                action.tooltip, action.stock_id)
        # don't set a zero-length short label
        if action.short_label:
            gtk_action.set_property('short-label', action.short_label)
        gtk_action.set_property('is-important', action.is_important)
        gtk_action_group = self.get_data('gtk_action_group')
        gtk_action_group.add_action_with_accel(gtk_action, action.accelerator)

    def remove_action(self, action):
        del self._actions[action.get_name()]
        self._remove_gtk_action(action.name)
        self.emit('remove-action', action)

    def _remove_gtk_action(self, name):
        gtk_action_group = self.get_data('gtk_action_group')
        gtk_action = gtk_action_group.get_action(name)
        gtk_action_group.remove_action(gtk_action)

    def update_action(self, action, old_name):
        self._remove_gtk_action(old_name)
        self._add_gtk_action(action)
        del self._actions[old_name]
        self._actions[action.name] = action
        if self._uimanager is not None:
            # we need to remove it and then add it to the uimanager
            # so all the proxies are updated
            gtk_action_group = self.get_data('gtk_action_group')
            self._uimanager.remove_action_group(gtk_action_group)
            self._uimanager.insert_action_group(gtk_action_group, 0)

    def get_action(self, action_name):
        return self._actions.get(action_name)

    def get_actions(self):
        return list(self._actions.values())

    def get_action_names(self):
        return list(self._actions.keys())

type_register(GActionGroup)




from livesettings import *
from django.utils.translation import ugettext_lazy as _

# this is so that the translation utility will pick up the string
gettext = lambda s: s

PAYMENT_GROUP = ConfigurationGroup('PAYMENT_COD', 
    _('COD Payment Module Settings'), 
    ordering = 100)

config_register_list(
    BooleanValue(PAYMENT_GROUP, 
        'SSL', 
        description=_("Use SSL for the module checkout pages?"), 
        default=False),
        
    BooleanValue(PAYMENT_GROUP, 
        'LIVE', 
        description=_("Accept real payments"),
        help_text=_("False if you want to be in test mode"),
        default=False),

        
    ModuleValue(PAYMENT_GROUP,
        'MODULE',
        description=_('Implementation module'),
        hidden=True,
        default = 'payment.modules.cod'),
        
    StringValue(PAYMENT_GROUP,
        'KEY',
        description=_("Module key"),
        hidden=True,
        default = 'COD'),

    StringValue(PAYMENT_GROUP,
        'LABEL',
        description=_('English name for this group on the checkout screens'),
        default = 'COD Payment',
        help_text = _('This will be passed to the translation utility')),

    StringValue(PAYMENT_GROUP,
        'URL_BASE',
        description=_('The url base used for constructing urlpatterns which will use this module'),
        default = '^cod/'),
        
    BooleanValue(PAYMENT_GROUP,
        'EXTRA_LOGGING',
        description=_("Verbose logs"),
        help_text=_("Add extensive logs during post."),
        default=False)
)



from wsgiref.headers import Headers as HeaderDict
from test_support import (
    unittest,
    ResponseClass,
    RequestClass,
    TestClient,
    WSGIController,
    WSGIControllerTestCase
    )

class HTTPResponseTestCase(WSGIControllerTestCase):
    charset = 'utf-8'
    mimetype = 'text/html'
    content = bytes()
    clientClass = TestClient

    def setUp(self):

        self.wsgi_app = ResponseClass(self.content, \
            mimetype=self.mimetype, charset=self.charset)
        
        #self.client = self.clientClass(self.callback)
 

class WSGIResponseTestCase(HTTPResponseTestCase):    
    def test_app_iter(self):
        response = self.wsgi_app
        self.assertEqual(len(response.content) == 0, True)
        self.assertEqual(len(response), 0)

    def test_write_unicode(self):
        # Python (2.5, 2.6, 2.7) specific test
        response = self.wsgi_app
        response.write(bytes('<p>hello, world!</p>'))
        self.assertEqual(response.content, str('<p>hello, world!</p>'))

    def test_format_status_int(self):    
        response = self.wsgi_app
        self.assertEqual(response.status_int, 200)
        self.assertEqual(response.status_code, '200 OK')
    
    def test_response_charset(self):  
        response = self.wsgi_app  
        self.assertEqual(response.charset==self.charset, True)

    def test_response_headers(self):
        response = self.wsgi_app
        self.assertEqual(isinstance(response.headers, list), True)

    def test_content_type(self):
        response = self.wsgi_app
        headers = HeaderDict(response.headers)
        content_type = headers['content-type']
        ct = response.content_type

        # Content-Type must match
        self.assertEqual(ct==content_type, True, ct)
        self.assertEqual(isinstance(ct, str), True, ct)
    
    def test_content_length(self):
        response = self.wsgi_app
        response.write(b'hello, world!')
        cl = response.content_length
        self.assertEqual(isinstance(cl, int), True, cl)
        self.assertEqual(int(cl), len(response.content), cl)
    
    def test_content_encoding(self):
        response = self.wsgi_app
        self.assertEqual(response.charset==self.charset, True, response.charset)

    #def test_get_response(self):
    #    response = self.client.get('/')
    #    self.assertEqual(response.status_int==200, True, response.status_int) 



from gazpacho.interfaces import IReferencable

class ReferenceContainer:
    """This class is intended to be used by gadgets to keep track of
    objects that keeps references to it.

    The objects that keep references to the gadgets are called
    referrers.  When the gadget is remvoed in some way the reference
    container makes it possible to update all of these referrers and
    clear the references. In the same way, when the removal is undone
    the references can be restored.

    When a referrer adds a reference to the gadget it should add
    itself to the reference container for that gadget. When the gadget
    is deleted or a deleted gadget is restored the contaner will make
    sure that the appropriate methods are called on the referrer. The
    referrer has to implement the IReferencable interface.
    """

    def __init__(self, gadget):
        """Initialize the reference container for a specific gadget.

        @param gadget: the gadget
        @type gadget: L{gazpacho.gadget.Gadget}
        """
        self._gadget = gadget
        self._referrers = []

    def clear_references(self):
        """Clear all references to this gadget.

        This method should be called when the gadget is deleted.
        """
        for referrer in self._referrers:
            referrer.remove_reference(self._gadget)

    def restore_references(self):
        """Restore all references that have previously been cleared.

        Should be called when a deleted gadget is restored.
        """
        for referrer in self._referrers:
            referrer.add_reference(self._gadget)

    def add_referrer(self, referrer):
        """Add a referrer, i.e. the one that referres to the gadget
        represented by this container.

        The has to implement the IReferencable interface which
        provides methods for removing and restoring the reference to
        the gadget.
        
        @param referrer: the object that referres to the gadget
        @type referrer: L{gazpacho.interfaces.IReferancable}
        """
        if not IReferencable.providedBy(referrer):
            raise TypeError("%s should be implementing %s" % (referrer, IReferencable))
        self._referrers.append(referrer)

    def remove_referrer(self, referrer):
        """Remove the specified referrer from the reference container.

        @param referrer: the object that referres to the gadget
        @type referrer: L{gazpacho.interfaces.IReferancable}
        """
        self._referrers.remove(referrer)



class BaseShipper(object):
    def __init__(self, cart=None, contact=None):
        self._calculated = False
        self.cart = cart
        self.contact = contact 
        self._calculated = False
        
        if cart or contact:
            self.calculate(cart, contact)
        
    def calculate(self, cart, contact):
        """
        Perform shipping calculations, separated from __init__ so that the object can be 
        used for keys and labels more easily.
        """
        self.cart = cart
        self.contact = contact
        self._calculated = True


from satchmo_settings import get_satchmo_setting
import logging

log = logging.getLogger('satchmo_store.shop')

if get_satchmo_setting('MULTISHOP'):
    log.debug('patching for multishop')
    from threaded_multihost import multihost_patch



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from django.conf.urls.defaults import *

#handle200 = 'notmm.lib.wsgiapp.render_to_response'

urlpatterns = patterns('',
        # Frontdoor (a simple generic view)
        (r'^$', 'sandbox.views.index', {
            'template_name' : 'sandbox/test.mako'
        }),
        (r'^croak$', 'sandbox.views.croak'),
        # with_session decorator test
        (r'^test$', 'sandbox.views.test_session', {
            'template_name' : 'sandbox/test.mako'
        }),
        # with_schevo_database test
        (r'^test.schevo$', 'sandbox.views.test_schevo_database', {
            'template_name' : 'sandbox/test.mako'
        }),
        (r'^moved/$', 'sandbox.handlers.handle302', {
            'location' : '/index.html'
        })
)




from keyedcache import cache_get, cache_set, NotCachedError
from product.models import Product
import logging

log = logging.getLogger('product.queries')

def bestsellers(count):
    """Look up the bestselling products and return in a list"""        
    sellers = []
    cached = False
    try:
        pks = cache_get('BESTSELLERS', count=count)
        if pks:
            pks = [long(pk) for pk in pks.split(',')]
            productdict = Product.objects.in_bulk(pks)
            #log.debug(productdict)
            for pk in pks:
                try:
                    if (int(pk)) in productdict:
                        key = int(pk)
                    elif long(pk) in productdict:
                        key = long(pk)
                    else:
                        continue
                    sellers.append(productdict[key])
                except ValueError:
                    pass
            
            log.debug('retrieved bestselling products from cache')
        cached = True    
    except NotCachedError:
        pass
    
    except ValueError:
        pass

    if not cached:
        products = Product.objects.active_by_site()
        work = []
        for p in products:
            ct = p.orderitem_set.count()
            if ct>0:
                work.append((ct, p))
        
        work.sort()
        work = work[-count:]
        work.reverse()
    
        sellers = []
        pks = []
    
        for p in work:
            product = p[1]
            pks.append("%i" % product.pk)
            sellers.append(product)
         
        pks = ",".join(pks)
        log.debug('calculated bestselling %i products, set to cache: %s', count, pks)
        cache_set('BESTSELLERS', count=count, value=pks)
        
    return sellers



"""Satchmo product signals

Signals:
 - `newsletter_subscription_updated`: Usage::
    
    Usage: newsletter_subscription_updated.send(sender, old_state=bool, new_state=bool, contact=contact) 
 
"""

import django.dispatch

newsletter_subscription_updated = django.dispatch.Signal()



"""Set of missing GTK+ macros and functions"""
# Copyright (C) 2006 by Nokia Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gtk

def widget_realized(widget):
    """Equivalent to the GTK_WIDGET_REALIZED macro"""
    return (widget.flags() & gtk.REALIZED) != 0

def widget_visible(widget):
    """Equivalent to the GTK_WIDGET_VISIBLE macro"""
    return (widget.flags() & gtk.VISIBLE) != 0

def widget_mapped(widget):
    """Equivalent to the GTK_WIDGET_MAPPED macro"""
    return (widget.flags() & gtk.MAPPED) != 0

def widget_drawable(widget):
    """Equivalent to the GTK_WIDGET_DRAWABLE macro"""
    return widget_visible(widget) and widget_mapped(widget)

def widget_sensitive(widget):
    """Equivalent to the GTK_WIDGET_SENSITIVE macro"""
    return (widget.flags() & gtk.SENSITIVE) != 0

def widget_parent_sensitive(widget):
    """Equivalent to the GTK_WIDGET_PARENT_SENSITIVE macro"""
    return (widget.flags() & gtk.PARENT_SENSITIVE) != 0

def widget_is_sensitive(widget):
    """Equivalent to the GTK_WIDGET_IS_SENSITIVE macro"""
    return widget_sensitive(widget) and widget_parent_sensitive(widget)

def widget_toplevel(widget):
    """Equivalent to the GTK_WIDGET_TOPLEVEL macro"""
    return (widget.flags() & gtk.TOPLEVEL) != 0

def widget_no_window(widget):
    """Equivalent to the GTK_WIDGET_NO_WINDOW macro"""
    return (widget.flags() & gtk.NO_WINDOW) != 0

def get_event_widget(event):
    """Equivalent to the gtk_get_event_widget function"""
    if (event and event.window):
        return event.window.get_user_data()

def clamp(value, low, high):
    """Return value limited to the range [low,hign]"""
    if value < low:
        return low
    if value > high:
        return high
    return value


def popup_grab_on_window(window, activate_time, grab_keyboard):
    """Do a grab on the gdk window"""
    mask = (gtk.gdk.BUTTON_PRESS_MASK | gtk.gdk.BUTTON_RELEASE_MASK |
            gtk.gdk.ENTER_NOTIFY_MASK | gtk.gdk.LEAVE_NOTIFY_MASK |
            gtk.gdk.POINTER_MOTION_MASK)
    if (gtk.gdk.pointer_grab(window, True, mask, None, None, activate_time) ==
        gtk.gdk.GRAB_SUCCESS):
        if (not grab_keyboard or
            gtk.gdk.keyboard_grab(window, True, activate_time) ==
            gtk.gdk.GRAB_SUCCESS):
            return True
        else:
            display = window.get_display()
            display.pointer_ungrab(activate_time)
            return False

    return False



# -*- coding: utf-8 -*-
""" threadlocals Middleware, provides a better, faster way to get at request and user.

:Authors:
   - Herbert Poul http://sct.sphene.net
   - Bruce Kroeze

Branched from [http://code.djangoproject.com/wiki/CookBookthreadlocalsAndUser CookBookThreadLocalsAndUser]
as modified by [http://sct.sphene.net Sphene Community tools].
"""

from threaded_multihost.threadlocals import set_thread_variable, set_current_user

class ThreadLocalMiddleware(object):
    """Middleware that gets various objects from the
    request object and saves them in thread local storage."""

    def process_request(self, request):
        set_thread_variable('request', request)
        set_current_user(request.user)
        


#!/usr/bin/env python

import sqlalchemy
import sandbox

from pkg_resources import resource_filename
from sqlalchemy.orm              import scoped_session, sessionmaker

from notmm.utils.django_settings import LazySettings

# init the local application data store
app_conf = sandbox.global_conf['sandbox']

# init the settings module (thread-local)
settings = LazySettings()

# SQLAlchemy settings
backend = app_conf['backend']
schema = backend + ':///' + resource_filename('sandbox', app_conf['path'])

engine = sqlalchemy.create_engine(schema, echo=app_conf['echo'])
metadata = sqlalchemy.MetaData(engine)
Session = scoped_session(sessionmaker(bind=engine))




"""A Mailman newsletter subscription interface.

To use this plugin, enable the newsletter module and set the newsletter module and name settings
in the admin settings page.
"""

from django.utils.translation import ugettext as _
from Mailman import MailList, Errors
from models import Subscription
from livesettings import config_value
import logging
import sys

log = logging.getLogger('newsletter.mailman')

class UserDesc: pass

def is_subscribed(contact):
    return Subscription.email_is_subscribed(contact.email)

def update_contact(contact, subscribe, attributes={}):
    email = contact.email
    current = Subscription.email_is_subscribed(email)
    attributesChanged = False
    sub = None
    
    if attributes:
        sub, created = Subscription.objects.get_or_create(email=email)
        if created:
            attributesChanged = True
        else:
            oldAttr = [(a.name,a.value) for a in sub.attributes.all()]
            oldAttr.sort()
        sub.update_attributes(attributes)
        newAttr = [(a.name,a.value) for a in sub.attributes.all()]
        newAttr.sort()
        
        if not created:
            attributesChanged = oldAttr != newAttr    

    if current == subscribe:
        if subscribe:
            if attributesChanged:
                result = _("Updated subscription for %(email)s.")
            else:
                result = _("Already subscribed %(email)s.")
        else:
            result = _("Already removed %(email)s.")
                
    else:
        if not sub:
            sub, created = Subscription.objects.get_or_create(email=email)
        sub.subscribed = subscribe
        sub.save()

        if subscribe:
            mailman_add(contact)
            result = _("Subscribed: %(email)s")
        else:
            mailman_remove(contact)
            result = _("Unsubscribed: %(email)s")
            

    return result % { 'email' : email }

def mailman_add(contact, listname=None, send_welcome_msg=None, admin_notify=None):
    """Add a Satchmo contact to a mailman mailing list.

    Parameters:
        - `Contact`: A Satchmo Contact
        - `listname`: the Mailman listname, defaulting to whatever you have set in settings.NEWSLETTER_NAME
        - `send_welcome_msg`: True or False, defaulting to the list default
        - `admin_notify`: True of False, defaulting to the list default
    """
    mm, listname = _get_maillist(listname)
    print >> sys.stderr, 'mailman adding %s to %s' % (contact.email, listname)

    if send_welcome_msg is None:
        send_welcome_msg = mm.send_welcome_msg

    userdesc = UserDesc()
    userdesc.fullname = contact.full_name
    userdesc.address = contact.email
    userdesc.digest = False

    if mm.isMember(contact.email):
        print >> sys.stderr, _('Already Subscribed: %s' % contact.email)

    else:
        try:
            try:
                mm.Lock()
                mm.ApprovedAddMember(userdesc, send_welcome_msg, admin_notify)
                mm.Save()
                print >> sys.stderr, _('Subscribed: %(email)s') % { 'email' : contact.email }

            except Errors.MMAlreadyAMember:
                print >> sys.stderr, _('Already a member: %(email)s') % { 'email' : contact.email }

            except Errors.MMBadEmailError:
                if userdesc.address == '':
                    print >> sys.stderr, _('Bad/Invalid email address: blank line')
                else:
                    print >> sys.stderr, _('Bad/Invalid email address: %(email)s') % { 'email' : contact.email }

            except Errors.MMHostileAddress:
                print >> sys.stderr, _('Hostile address (illegal characters): %(email)s') % { 'email' : contact.email }

        finally:
            mm.Unlock()

def mailman_remove(contact, listname=None, userack=None, admin_notify=None):
    """Remove a Satchmo contact from a Mailman mailing list

    Parameters:
        - `contact`: A Satchmo contact
        - `listname`: the Mailman listname, defaulting to whatever you have set in settings.NEWSLETTER_NAME
        - `userack`: True or False, whether to notify the user, defaulting to the list default
        - `admin_notify`: True or False, defaulting to the list default
    """


    mm, listname = _get_maillist(listname)
    print >> sys.stderr, 'mailman removing %s from %s' % (contact.email, listname)

    if mm.isMember(contact.email):
        try:
            mm.Lock()
            mm.ApprovedDeleteMember(contact.email, 'satchmo_ext.newsletter',  admin_notify, userack)
            mm.Save()
        finally:
            mm.Unlock()

def _get_maillist(listname):
    try:
        if not listname:
            listname = config_value('NEWSLETTER', 'NEWSLETTER_NAME')

        if listname == "":
            log.warn("NEWSLETTER_NAME not set in store settings")
            raise NameError('No NEWSLETTER_NAME in settings')

        return MailList.MailList(listname, lock=0), listname

    except Errors.MMUnknownListError:
        print >> sys.stderr, "Can't find the MailMan newsletter: %s" % listname
        raise NameError('No such newsletter, "%s"' % listname)



from django.conf import settings
from django.contrib.auth import login, REDIRECT_FIELD_NAME
from django.contrib.auth.forms import AuthenticationForm
from django.contrib.auth.models import User
from django.contrib.sites.models import Site, RequestSite
from django.core import urlresolvers
from django.http import HttpResponseRedirect, QueryDict
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils.translation import ugettext_lazy as _
from django.views.decorators.cache import never_cache
from forms import RegistrationAddressForm, RegistrationForm, EmailAuthenticationForm
from l10n.models import Country
from livesettings import config_get_group, config_value
from satchmo_store.accounts.mail import send_welcome_email
from satchmo_store.accounts import signals
from satchmo_store.contact import CUSTOMER_ID
from satchmo_store.contact.models import Contact
from satchmo_store.shop.models import Config, Cart

import logging

log = logging.getLogger('satchmo_store.accounts.views')

YESNO = (
    (1, _('Yes')),
    (0, _('No'))
)

def emaillogin(request, template_name='registration/login.html', 
    auth_form=EmailAuthenticationForm, redirect_field_name=REDIRECT_FIELD_NAME):
    "Displays the login form and handles the login action. Altered to use the EmailAuthenticationForm"

    redirect_to = request.REQUEST.get(redirect_field_name, '')

    # Avoid redirecting to logout if the user clicked on login after logout
    if redirect_to == urlresolvers.reverse('auth_logout'):
        redirect_to = None

    success, todo = _login(request, redirect_to)
    if success:
        # return the response redirect
        return todo
    else:
        # continue with the login form
        form = todo

    request.session.set_test_cookie()
    if Site._meta.installed:
        current_site = Site.objects.get_current()
    else:
        current_site = RequestSite(request)

    return render_to_response(template_name, {
        'form': form,
        redirect_field_name: redirect_to,
        'site_name': current_site.name,
    }, context_instance=RequestContext(request))
emaillogin = never_cache(emaillogin)

def _login(request, redirect_to, auth_form=EmailAuthenticationForm):
    """"Altered version of the default login, intended to be called by `combined_login`.

    Returns tuple:
    - success
    - redirect (success) or form (on failure)
    """
    
    if request.method == 'POST':
        form = auth_form(data=request.POST)
        if form.is_valid():
            # Light security check -- make sure redirect_to isn't garbage.
            if not redirect_to or '//' in redirect_to or ' ' in redirect_to:
                redirect_to = settings.LOGIN_REDIRECT_URL
            login(request, form.get_user())
            if request.session.test_cookie_worked():
                request.session.delete_test_cookie()
                if config_value('SHOP','PERSISTENT_CART'):
                    _get_prev_cart(request)
            return (True, HttpResponseRedirect(redirect_to))
        else:
            log.debug(form.errors)
    else:
        form = auth_form(request)

    return (False, form)

def _get_prev_cart(request):
    try:
        contact = request.user.contact_set.get()
        saved_cart = contact.cart_set.latest('date_time_created')
        # If the latest cart has len == 0, cart is unusable.
        if len(saved_cart) and request.session['cart']:
            # Merge the two carts together
            existing_cart = Cart.objects.from_request(request, create=False)
            saved_cart.merge_carts(existing_cart)
            request.session['cart'] = saved_cart.id
    except Exception, e:
        pass


def register_handle_address_form(request, redirect=None):
    """
    Handle all registration logic.  This is broken out from "register" to allow easy overriding/hooks
    such as a combined login/register page.
    
    This handler allows a login or a full registration including address.

    Returns:
    - Success flag
    - HTTPResponseRedirect (success) or form (fail)
    - A dictionary with extra context fields
    """

    shop = Config.objects.get_current()
    try:
        contact = Contact.objects.from_request(request)
    except Contact.DoesNotExist:
        contact = None

    if request.method == 'POST':

        form = RegistrationAddressForm(request.POST, shop=shop, contact=contact)

        if form.is_valid():
            contact = form.save(request)

            if not redirect:
                redirect = urlresolvers.reverse('registration_complete')
            return (True, HttpResponseRedirect(redirect))

    else:
        initial_data = {}
        if contact:
            initial_data = {
                'email': contact.email,
                'first_name': contact.first_name,
                'last_name': contact.last_name }
            address = contact.billing_address
            if address:
                initial_data['street1'] = address.street1
                initial_data['street2'] = address.street2
                initial_data['state'] = address.state
                initial_data['city'] = address.city
                initial_data['postal_code'] = address.postal_code
                try:
                    initial_data['country'] = address.country
                except Country.DoesNotExist:
                    USA = Country.objects.get(iso2_code__exact="US")    
                    initial_data['country'] = USA

        form = RegistrationAddressForm(initial=initial_data, shop=shop, contact=contact)

    return (False, form, {'country' : shop.in_country_only})


def register_handle_form(request, redirect=None):
    """
    Handle all registration logic.  This is broken out from "register" to allow easy overriding/hooks
    such as a combined login/register page.
    
    This method only presents a typical login or register form, not a full address form 
    (see register_handle_address_form for that one.)

    Returns:
    - Success flag
    - HTTPResponseRedirect (success) or form (fail)
    """

    if request.method == 'POST':
        form = RegistrationForm(request.POST)
        if form.is_valid():
            contact = form.save(request)

            # look for explicit "next"
            next = request.POST.get('next', '')
            if not next:
                if redirect:
                    next = redirect
                else:
                    next = urlresolvers.reverse('registration_complete')
            return (True, HttpResponseRedirect(next))

    else:
        initial_data = {}
        try:
            contact = Contact.objects.from_request(request, create=False)
            initial_data = {
                'email': contact.email,
                'first_name': contact.first_name,
                'last_name': contact.last_name,
            }
        except Contact.DoesNotExist:
            log.debug("No contact in request")
            contact = None
            
        initial_data['next'] = request.GET.get('next', '') 

        form = RegistrationForm(initial=initial_data)

    return (False, form)



#---- Views

def activate(request, activation_key):
    """
    Activates a user's account, if their key is valid and hasn't
    expired.
    """
    from registration.models import RegistrationProfile
    activation_key = activation_key.lower()
    account = RegistrationProfile.objects.activate_user(activation_key)

    if account:
        # ** hack for logging in the user **
        # when the login form is posted, user = authenticate(username=data['username'], password=data['password'])
        # ...but we cannot authenticate without password... so we work-around authentication
        account.backend = settings.AUTHENTICATION_BACKENDS[0]
        _login(request, account)
        contact = Contact.objects.get(user=account)
        request.session[CUSTOMER_ID] = contact.id
        send_welcome_email(contact.email, contact.first_name, contact.last_name)
        signals.satchmo_registration_verified.send(contact, contact=contact)

    context = RequestContext(request, {
        'account': account,
        'expiration_days': config_value('SHOP', 'ACCOUNT_ACTIVATION_DAYS'),
    })
    return render_to_response('registration/activate.html',
                              context_instance=context)


def login_signup(request, template_name="contact/login_signup.html", registration_handler=register_handle_form):
    """Display/handle a combined login and create account form"""

    redirect_to = request.REQUEST.get(REDIRECT_FIELD_NAME, '')

    loginform = None
    createform = None
    extra_context = {}

    if request.POST:
        action = request.POST.get('action', 'login')
        if action == 'create':
            #log.debug('Signup form')
            ret = registration_handler(request, redirect=redirect_to)
            success = ret[0]
            todo = ret[1]
            if len(ret) > 2:
                extra_context = ret[2]

            if success:
                #log.debug('Successful %s form submit, sending to reg complete page')
                if redirect_to:
                    return HttpResponseRedirect(redirect_to)
                else:
                    ctx = RequestContext(request, {
                        REDIRECT_FIELD_NAME: redirect_to,
                    })

                    return render_to_response('registration/registration_complete.html',
                                              context_instance=ctx)
            else:
                createform = todo

        else:
            #log.debug('Login form')
            success, todo = _login(request, redirect_to)
            if success:
                return todo
            else:
                loginform = todo

        request.POST = QueryDict("")
        

    else:
        request.session.set_test_cookie()

    if not loginform:
        success, loginform = _login(request, redirect_to)
    if not createform:
        ret = registration_handler(request, redirect_to)
        success = ret[0]
        createform = ret[1]
        if len(ret) > 2:
            extra_context = ret[2]

    site = Site.objects.get_current()

    if config_get_group('NEWSLETTER'):
        show_newsletter = True
    else:
        show_newsletter = False


    ctx = {
        'loginform': loginform,
        'createform' : createform,
        REDIRECT_FIELD_NAME: redirect_to,
        'site_name': site.name,
        'show_newsletter' : show_newsletter,
    }

    if extra_context:
        ctx.update(extra_context)            

    context = RequestContext(request, ctx)

    return render_to_response(template_name, context_instance=context)


def login_signup_address(request, template_name="contact/login_signup_address.html"):
    """
    View which allows a user to login or else fill out a full address form.
    """
    return login_signup(request, template_name=template_name, registration_handler=register_handle_address_form)


def register(request, redirect=None, template='registration/registration_form.html'):
    """
    Allows a new user to register an account.
    """

    ret = register_handle_form(request, redirect)
    success = ret[0]
    todo = ret[1]
    if len(ret) > 2:
        extra_context = ret[2]
    else:
        extra_context = {}

    if success:
        return todo
    else:
        if config_get_group('NEWSLETTER'):
            show_newsletter = True
        else:
            show_newsletter = False

        ctx = {
            'form': todo, 
            'title' : _('Registration Form'),
            'show_newsletter' : show_newsletter
        }

        if extra_context:
            ctx.update(extra_context)

        context = RequestContext(request, ctx)
        return render_to_response(template, context_instance=context)




# By Neum Schmickrath 
# www.pageworthy.com
from django.utils.translation import ugettext_lazy as _
from livesettings import *

SHIP_MODULES = config_get('SHIPPING', 'MODULES')
SHIP_MODULES.add_choice(('shipping.modules.fedex', 'FEDEX'))

SHIPPING_GROUP = ConfigurationGroup('shipping.modules.fedex',
  _('FedEx Shipping Settings'),
  requires = SHIP_MODULES,
  requiresvalue='shipping.modules.fedex',
  ordering = 101
)

config_register_list(

    StringValue(SHIPPING_GROUP,
        'METER_NUMBER',
        description=_('FedEx Meter Number'),
        help_text=_('Meter Number provided by FedEx.'),
        default=u''),
    
    StringValue(SHIPPING_GROUP,
        'ACCOUNT',
        description=_('FedEx Account Number'),
        help_text=_('FedEx Account Number.'),
        default=u''),
    
    MultipleStringValue(SHIPPING_GROUP,
        'SHIPPING_CHOICES',
        description=_('FedEx Shipping Choices Available to customers.'),
        choices = (
                    (('PRIORITYOVERNIGHT','Priority Overnight')),
                    (('STANDARDOVERNIGHT','Standard Overnight')),
                    (('FIRSTOVERNIGHT', 'First Overnight')),
                    (('FEDEX2DAY','2 Day')),
                    (('FEDEXEXPRESSSAVER','Express Saver')),
                    (('INTERNATIONALPRIORITY','International Priority')),
                    (('INTERNATIONALECONOMY', 'International Economy')),
                    (('INTERNATIONALFIRST', 'International First')),
                    (('FEDEX1DAYFREIGHT', '1 Day Freight')),
                    (('FEDEX2DAYFREIGHT', '2 Day Freight')),
                    (('FEDEX3DAYFREIGHT', '3 Day Freight')),
                    (('FEDEXGROUND','Ground')),
                    (('GROUNDHOMEDELIVERY', 'Ground Home Delivery')),
                    (('INTERNATIONALPRIORITYFREIGHT', 'International Priority Freight')),
                    (('INTERNATIONALECONOMYFREIGHT','International Economy Freight')),
                    (('EUROPEFIRSTINTERNATIONALPRIORITY', 'Europe International Priority')),

        ),
        default = ('FEDEXGROUND',)),

    StringValue(SHIPPING_GROUP,
        'SHIPPING_PACKAGE',
        description = _('Type of container/package used to ship product.'),
        choices = (
                    (('FEDEXENVELOPE','FEDEXENVELOPE')),
                    (('FEDEXPAK','FEDEXPAK')),
                    (('FEDEXBOX','FEDEXBOX')),
                    (('FEDEXTUBE','FEDEXTUBE')),
                    (('FEDEX10KGBOX','FEDEX10KGBOX')),
                    (('FEDEX25KGBOX','FEDEX25KGBOX')),
                    (('YOURPACKAGING','YOURPACKAGING')),
        ),
        default = u'FEDEXENVELOPE'),

    BooleanValue(SHIPPING_GROUP,
        'LIVE',
        description=_('Access production FedEx server'),
        help_text=_('Use this when your store is in production.'),
        default=False),

    StringValue(SHIPPING_GROUP,
        'CONNECTION',
        description=_('Submit to URL'),
        help_text=_('Address to submit live transactions.'),
        default='https://gateway.fedex.com/GatewayDC'),

    StringValue(SHIPPING_GROUP,
        'CONNECTION_TEST',
        description=_('Submit to TestURL'),
        help_text=_('Address to submit test transactions.'),
        default='https://gatewaybeta.fedex.com/GatewayDC'),
    
    BooleanValue(SHIPPING_GROUP,
        'SINGLE_BOX',
        description=_("Single Box?"),
        help_text=_("Use just one box and ship by weight?  If no then every item will be sent in its own box."),
        default=True),

    BooleanValue(SHIPPING_GROUP,
        'VERBOSE_LOG',
        description=_("Verbose logs"),
        help_text=_("Send the entire request and response to the log - for debugging help when setting up FedEx."),
        default=False)
)



import os
from test_support import unittest

from notmm.datastore import threadlocal

LocalStore = threadlocal.ThreadLocalStore

class ThreadLocalStoreTestCase(unittest.TestCase):
    
    def setUp(self):
        self.modname = os.environ.get('DJANGO_SETTINGS_MODULE', 'local_settings')
        self.storable = LocalStore(self.modname)

    def test_loads(self):
        # loads the objects by doing a __import__ 
        
        self.storable.loads()
        self.assertEqual(self.storable.initialized, True)
        self.assertEqual(hasattr(self.storable, 'DEBUG'), True)

    def test_count(self):
        # return the total amount of valid objects found
        # in self.storable. 
        self.storable = LocalStore(self.modname, autoload=True)
        settings_count = self.storable.count() 
        self.assertEqual(isinstance(settings_count, int), True)
    
    def test_clear(self):
        # for zeroing the settings found in __dict__...

        self.storable.clear()
        self.assertEqual(self.storable.count(), 0)
        #self.assertEqual(self.storable.initialized, False)

    def tearDown(self):
        pass
    def runTest(self):
        pass




import os
import urllib
from django.conf import settings
from django.contrib.admin.views.decorators import staff_member_required
from django.http import HttpResponse, HttpResponseRedirect
from django.template import loader, Context
from django.shortcuts import get_object_or_404
from django.utils.encoding import smart_str
from django.views.decorators.cache import never_cache
from satchmo_store.shop.models import Order
from satchmo_store.shop.models import Config
from livesettings import config_value

def displayDoc(request, id, doc):
    import trml2pdf
    # Create the HttpResponse object with the appropriate PDF headers for an invoice or a packing slip
    order = get_object_or_404(Order, pk=id)
    shopDetails = Config.objects.get_current()
    filename_prefix = shopDetails.site.domain
    if doc == "invoice":
        filename = "%s-invoice.pdf" % filename_prefix
        template = "invoice.rml"
    elif doc == "packingslip":
        filename = "%s-packingslip.pdf" % filename_prefix
        template = "packing-slip.rml"
    elif doc == "shippinglabel":
        filename = "%s-shippinglabel.pdf" % filename_prefix
        template = "shipping-label.rml"
    else:
        return HttpResponseRedirect('/admin')
    response = HttpResponse(mimetype='application/pdf')
    response['Content-Disposition'] = 'attachment; filename=%s' % filename
    icon_uri = config_value('SHOP', 'LOGO_URI')
    t = loader.get_template(os.path.join('shop/pdf', template))
    c = Context({
                'filename' : filename,
                'iconURI' : icon_uri,
                'shopDetails' : shopDetails,
                'order' : order,
                })
    pdf = trml2pdf.parseString(smart_str(t.render(c)))
    response.write(pdf)
    return response
displayDoc = staff_member_required(never_cache(displayDoc))




import datetime
from django.contrib.auth.decorators import user_passes_test
from django.core import urlresolvers
from django.http import Http404
from django.shortcuts import get_object_or_404, render_to_response
from payment.config import credit_choices
from product.models import Product, Category
from satchmo_store.shop.models import Config
from django.utils.translation import ugettext_lazy as _

@user_passes_test(lambda u: u.is_authenticated() and u.is_staff, login_url='/accounts/login/')
def admin_product_feed(request, category=None, template="product_feeds/product_feed.csv", mimetype="text/csv"):
    """Admin authenticated feed - same as product feed but for different types of feeds.
    """
    return product_feed(request, category=category, template=template, mimetype=mimetype)

def product_feed(request, category=None, template="product_feeds/googlebase_atom.xml", mimetype="application/atom+xml"):
    """Build a feed of all active products.
    """

    shop_config = Config.objects.get_current()
    if category:
        try:
            cat = Category.objects.active().get(slug=category)
            products = cat.active_products()
        except Category.DoesNotExist:
            raise Http404, _("Bad Category: %s" % category)
    else:
        cat = None
        products = Product.objects.active_by_site()
        
    products = filter(lambda product:"ConfigurableProduct" not in product.get_subtypes(), products)
    
    params = {}
    view = 'satchmo_atom_feed'
    if category:
        params['category'] = category
        view = 'satchmo_atom_category_feed'
    
    url = shop_config.base_url + urlresolvers.reverse(view, None, params)
    
    payment_choices = [c[1] for c in credit_choices(None, True)]
    
    return render_to_response(template, {
        'products' : products,
        'category' : cat,
        'url' : url,
        'shop' : shop_config,
        'payments' : payment_choices,
        'date' : datetime.datetime.now(),
        },
        mimetype=mimetype)



"""Sends mail related to accounts."""

from django.conf import settings
from django.template import Context
from django.template import loader
from django.utils.translation import ugettext
from satchmo_store.shop.models import Config
from socket import error as SocketError
import logging

__all__ = ['send_html_mail', 'send_welcome_email']

if "mailer" in settings.INSTALLED_APPS:
    from mailer import send_mail
else:
    from django.core.mail import send_mail

log = logging.getLogger('satchmo_store.accounts.mail')

def send_html_mail(*args, **kwargs):
    return send_mail(*args, **kwargs)

def send_welcome_email(email, first_name, last_name):
    """Send a store new account welcome mail to `email`."""
    
    t = loader.get_template('registration/welcome.txt')
    shop_config = Config.objects.get_current()
    shop_email = shop_config.store_email
    subject = ugettext("Welcome to %s") % shop_config.store_name
    c = Context({
        'first_name': first_name,
        'last_name': last_name,
        'company_name': shop_config.store_name,
        'site_url': shop_config.site.domain,
        'login_url': settings.LOGIN_URL,
    })
    body = t.render(c)
    try:
        send_mail(subject, body, shop_email, [email], fail_silently=False)
    except SocketError, e:
        if settings.DEBUG:
            log.error('Error sending mail: %s' % e)
            log.warn('Ignoring email error, since you are running in DEBUG mode.  Email was:\nTo:%s\nSubject: %s\n---\n%s', email, subject, body)
        else:
            log.fatal('Error sending mail: %s' % e)
            raise IOError('Could not send email, please check to make sure your email settings are correct, and that you are not being blocked by your ISP.')



from django import forms
from django.db.models.fields import DecimalField
import logging
from widgets import CurrencyWidget

log = logging.getLogger('satchmo_utils.fields')

class CurrencyField(DecimalField):
    
    def __init__(self, *args, **kwargs):
        self.places = kwargs.pop('display_decimal', 2)
        super(CurrencyField, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        defaults = {
            'max_digits': self.max_digits,
            'decimal_places': self.decimal_places,
            'form_class': forms.DecimalField,
            'widget': CurrencyWidget,
        }
        defaults.update(kwargs)
        return super(CurrencyField, self).formfield(**defaults)



from django.contrib.auth.decorators import login_required
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils.translation import ugettext_lazy as _
from satchmo_store.contact.models import Contact
from satchmo_store.shop.models import Order
from satchmo_utils.views import bad_or_missing
from livesettings import config_value 

def order_history(request):
    orders = None
    try:
        contact = Contact.objects.from_request(request, create=False)
        orders = Order.objects.filter(contact=contact).order_by('-time_stamp')
    
    except Contact.DoesNotExist:
        contact = None
        
    ctx = RequestContext(request, {
        'contact' : contact,
        'default_view_tax': config_value('TAX', 'DEFAULT_VIEW_TAX'),
        'orders' : orders})

    return render_to_response('shop/order_history.html', context_instance=ctx)

order_history = login_required(order_history)

def order_tracking(request, order_id):
    order = None
    try:
        contact = Contact.objects.from_request(request, create=False)
        try:
            order = Order.objects.get(id__exact=order_id, contact=contact)
        except Order.DoesNotExist:
            pass
    except Contact.DoesNotExist:
        contact = None

    if order is None:
        return bad_or_missing(request, _("The order you have requested doesn't exist, or you don't have access to it."))

    ctx = RequestContext(request, {
        'default_view_tax': config_value('TAX', 'DEFAULT_VIEW_TAX'),
        'contact' : contact,
        'order' : order})

    return render_to_response('shop/order_tracking.html', context_instance=ctx)

order_tracking = login_required(order_tracking)



from livesettings import config_value
from payment.listeners import capture_on_ship_listener
from product.models import Product
from product.listeners import default_product_search_listener, discount_used_listener
from satchmo_store.contact import signals as contact_signals
from satchmo_store.accounts.mail import send_html_mail
from satchmo_store.shop import signals
from satchmo_store.shop.exceptions import OutOfStockError
from satchmo_store.shop.models import Order
from signals_ahoy.signals import application_search

import notification
import logging

log = logging.getLogger('shop.listeners')

# --------------- Optional listeners
def only_one_item_in_cart(sender, cart=None, cartitem=None, **kwargs):
    for item in cart.cartitem_set.all():
        if not item == cartitem:
            log.debug('only one item in cart active: removing %s', item)
            item.delete()

# --------------- Default listeners

def decrease_inventory_on_sale(sender, order=None, **kwargs):
    """Track inventory and total sold."""
    # Added to track total sold for each product
    for item in order.orderitem_set.all():
        product = item.product
        product.total_sold += item.quantity
        if config_value('PRODUCT','TRACK_INVENTORY'):
            product.items_in_stock -= item.quantity
        product.save()

def recalc_total_on_contact_change(contact=None, **kwargs):
    """If the contact has changed, recalculate the order total to ensure all current triggers are hit."""
    #TODO: pull just the current order once we start using threadlocal middleware
    log.debug("Recalculating all contact orders not in process")
    orders = Order.objects.filter(contact=contact, status="")
    log.debug("Found %i orders to recalc", orders.count())
    for order in orders:
        order.copy_addresses()
        order.recalculate_total()

def remove_order_on_cart_update(request=None, cart=None, **kwargs):
    """Remove partially completed order when the cart is updated"""
    if request:
        log.debug("caught cart changed signal - remove_order_on_cart_update")
        Order.objects.remove_partial_order(request)

def veto_out_of_stock(sender, cartitem=None, added_quantity=0, **kwargs):
    """Listener which vetoes adding products to the cart which are out of stock."""

    if config_value('PRODUCT','NO_STOCK_CHECKOUT') == False:
        product = cartitem.product
        need_qty = cartitem.quantity + added_quantity
        if product.items_in_stock < need_qty:
            log.debug('out of stock on %s', product.slug)
            raise OutOfStockError(product, product.items_in_stock, need_qty)


def start_default_listening():
    """Add required default listeners"""
    contact_signals.satchmo_contact_location_changed.connect(recalc_total_on_contact_change, sender=None)
    signals.order_success.connect(decrease_inventory_on_sale)
    signals.order_success.connect(notification.order_success_listener, sender=None)
    signals.order_success.connect(discount_used_listener, sender=None)
    signals.satchmo_cart_changed.connect(remove_order_on_cart_update, sender=None)
    application_search.connect(default_product_search_listener, sender=Product)
    signals.satchmo_order_status_changed.connect(capture_on_ship_listener)
    signals.satchmo_order_status_changed.connect(notification.notify_on_ship_listener)
    signals.satchmo_cart_add_verify.connect(veto_out_of_stock)

    signals.sending_store_mail.connect(send_html_mail)

    log.debug('Added default shop listeners')



from livesettings import *
from django.utils.translation import ugettext_lazy as _

PAYMENT_GROUP = ConfigurationGroup('PAYMENT_PAYPAL', 
    _('Paypal Payment Module Settings'), 
    ordering = 101)

config_register_list(

StringValue(PAYMENT_GROUP,
    'CURRENCY_CODE',
    description=_('Currency Code'),
    help_text=_('Currency code for Paypal transactions.'),
    default = 'USD'), # XXX LiveStore: currency defined as settings.DEFAULT_CURRENCY
    
StringValue(PAYMENT_GROUP,
    'POST_URL',
    description=_('Post URL'),
    help_text=_('The Paypal URL for real transaction posting.'),
    default="https://www.paypal.com/cgi-bin/webscr"),

StringValue(PAYMENT_GROUP,
    'POST_TEST_URL',
    description=_('Post URL'),
    help_text=_('The Paypal URL for test transaction posting.'),
    default="https://www.sandbox.paypal.com/cgi-bin/webscr"),

StringValue(PAYMENT_GROUP,
    'BUSINESS',
    description=_('Paypal account email'),
    help_text=_('The email address for your paypal account'),
    default=""),

StringValue(PAYMENT_GROUP,
    'BUSINESS_TEST',
    description=_('Paypal test account email'),
    help_text=_('The email address for testing your paypal account'),
    default=""),

StringValue(PAYMENT_GROUP,
    'RETURN_ADDRESS',
    description=_('Return URL'),
    help_text=_('Where Paypal will return the customer after the purchase is complete.  This can be a named url and defaults to the standard checkout success.'),
    default="satchmo_checkout-success"),
    
BooleanValue(PAYMENT_GROUP, 
    'SSL', 
    description=_("Use SSL for the module checkout pages?"), 
    default=False),
    
BooleanValue(PAYMENT_GROUP, 
    'LIVE', 
    description=_("Accept real payments"),
    help_text=_("False if you want to be in test mode"),
    default=False),
    
ModuleValue(PAYMENT_GROUP,
    'MODULE',
    description=_('Implementation module'),
    hidden=True,
    default = 'payment.modules.paypal'),
    
StringValue(PAYMENT_GROUP,
    'KEY',
    description=_("Module key"),
    hidden=True,
    default = 'PAYPAL'),

StringValue(PAYMENT_GROUP,
    'LABEL',
    description=_('English name for this group on the checkout screens'),
    default = 'PayPal',
    help_text = _('This will be passed to the translation utility')),

StringValue(PAYMENT_GROUP,
    'URL_BASE',
    description=_('The url base used for constructing urlpatterns which will use this module'),
    default = '^paypal/'),
    
BooleanValue(PAYMENT_GROUP,
    'EXTRA_LOGGING',
    description=_("Verbose logs"),
    help_text=_("Add extensive logs during post."),
    default=True)
)

PAYMENT_GROUP['TEMPLATE_OVERRIDES'] = {
    'shop/checkout/confirm.html' : 'shop/checkout/paypal/confirm.html',
}



"""Utility functions used by signals to attach Ratings to Comments"""
from django.contrib.comments.models import Comment
from django.contrib.sites.models import Site
from django.core import urlresolvers
from django.utils.encoding import smart_str
from livesettings import config_value
from models import ProductRating
from product.models import Product
from satchmo_utils import url_join
import logging
from django.conf import settings

log = logging.getLogger('productratings')

def save_rating(comment=None, request=None, **kwargs):
    """Create a rating and save with the comment"""
    
    # should always be true
    if request.method != "POST":
        return
    
    data = request.POST.copy()
    if 'rating' not in data:
        return
    
    raw = data['rating']
    try:
        rating = int(raw)
    except ValueError:
        log.error('Could not parse rating from posted rating: %s', raw)
        return
        
    if comment.content_type.app_label == "product" and comment.content_type.model == "product":
        if hasattr(comment, 'rating'):
            log.debug('editing existing comment %s, setting rating=%i', comment, rating)
            productrating = comment.rating
            productrating.rating = rating
    
        else:
            log.debug("Creating new rating for comment: %s = %i", comment, rating)
            p = Product.objects.get(pk=comment.object_pk)
            productrating = ProductRating(comment=comment, rating=rating)
        
            productrating.save()
    else:
        log.debug('Not saving rating for comment on a %s object', comment.content_type.model)
    
def one_rating_per_product(comment=None, request=None, **kwargs):
    site = Site.objects.get_current()
    comments = Comment.objects.filter(object_pk__exact=comment.object_pk,
                               content_type__app_label__exact='product',
                               content_type__model__exact='product',
                               site__exact=site,
                               is_public__exact=True,
                               user__exact=request.user)
                               
    for c in comments:
        if not c == comment:
            c.delete()            

def check_with_akismet(comment=None, request=None, **kwargs):
    if config_value("PRODUCT", "AKISMET_ENABLE"):
        key = config_value("PRODUCT", "AKISMET_KEY")
        if key:             
            site = Site.objects.get_current()
            shop = urlresolvers.reverse('satchmo_shop_home')
            from akismet import Akismet
            akismet = Akismet(
                key=settings.AKISMET_API_KEY,
                blog_url='http://%s' % url_join(site.domain, shop))
            if akismet.verify_key():
                akismet_data = { 'comment_type': 'comment',
                                 'referrer': request.META.get('HTTP_REFERER', ""),
                                 'user_ip': comment.ip_address,
                                 'user_agent': '' }
                if akismet.comment_check(smart_str(comment.comment), data=akismet_data, build_data=True):
                    comment.is_public=False
                    comment.save()
                    log.info("Akismet marked comment #%i as spam", comment.id)
                else:
                    log.debug("Akismet accepted comment #%i", comment.id)
            else:
                log.warn("Akismet key '%s' not accepted by akismet service.", key)
        else:
            log.info("Akismet enabled, but no key found.  Please put in your admin settings.")



# Copyright (C) 2006 by Nokia Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gtk

from kiwi.component import get_utility
from kiwi.environ import environ

from gazpacho.config import config
from gazpacho.interfaces import IGazpachoApp, IPluginManager
from gazpacho.loader.loader import ObjectBuilder

(COL_ACTIVATED,
 COL_TITLE,
 COL_PLUGININFO) = list(range(3))

class PreferencesDialog:
    def __init__(self):
        self.app = get_utility(IGazpachoApp)
        self.plugin_manager = get_utility(IPluginManager)
        ui_file = environ.find_resource('glade', 'preferences.glade')
        app_window = self.app.get_window()

        self.ob = ObjectBuilder(ui_file)
        self.dialog = self.ob.get_widget('dialog')

        # dialog setup
        self.dialog.set_modal(True)
        self.dialog.set_transient_for(app_window)

        # this should go into the glade file as soon as we get support for it
        close = self.ob.get_widget('close')
        close.connect('clicked', self.on_close__clicked)

        # setup each tab
        self._setup_plugins_tab()

    def on_close__clicked(self, button):
        self.dialog.response(gtk.RESPONSE_CLOSE)

    def run(self):
        self.dialog.show()
        self.dialog.run()
        self.dialog.destroy()

    def _setup_plugins_tab(self):
        plugin_list = self.ob.get_widget('plugins_list')

        model = gtk.ListStore(bool, str, object)

        # fill the model
        for plugin_info in self.plugin_manager.get_plugins():
            activated = self.plugin_manager.is_activated(plugin_info.name)
            model.append((activated, plugin_info.title, plugin_info))

        plugin_list.set_model(model)

        # make the cells editable
        toggle = self.ob.get_widget('treeviewcolumn1-renderer1')
        toggle.set_property('activatable', True)
        toggle.connect('toggled', self.on_plugin__activated, model)

        # disable buttons
        plugin_about = self.ob.get_widget('plugin_about')
        plugin_about.set_sensitive(False)
        plugin_about.connect('clicked', self.on_plugin_about__clicked)

        plugin_preferences = self.ob.get_widget('plugin_preferences')
        plugin_preferences.set_sensitive(False)
        plugin_preferences.connect('clicked', self.on_plugin_prefs__clicked)

        selection = plugin_list.get_selection()
        selection.connect('changed', self.on_plugin_selection__changed)

    def on_plugin__activated(self, cell, path, model):
        row = model[path]
        new_value = not row[COL_ACTIVATED]
        row[COL_ACTIVATED] = new_value

        plugin = row[COL_PLUGININFO]

        if new_value:
            self.plugin_manager.activate_plugin(plugin.name, self.app)
        else:
            self.plugin_manager.deactivate_plugin(plugin.name)

        plugin_preferences = self.ob.get_widget('plugin_preferences')
        plugin_preferences.set_sensitive(new_value)

        config.set_plugin(plugin.name, new_value)

    def on_plugin_selection__changed(self, selection):
        model, model_iter = selection.get_selected()
        plugin_about = self.ob.get_widget('plugin_about')
        plugin_preferences = self.ob.get_widget('plugin_preferences')
        if model_iter:
            plugin_about.set_sensitive(True)
            activated = model.get_value(model_iter, COL_ACTIVATED)
            plugin_preferences.set_sensitive(activated)

        else:
            plugin_about.set_sensitive(False)
            plugin_preferences.set_sensitive(False)

    def on_plugin_about__clicked(self, button):
        plugin_info = self._get_selected_plugin()
        if plugin_info:
            dialog = gtk.AboutDialog()
            dialog.set_name(plugin_info.title)
            dialog.set_version(plugin_info.version)
            dialog.set_comments(plugin_info.description)
            dialog.set_authors([plugin_info.author])
            dialog.set_transient_for(self.dialog)
            dialog.run()
            dialog.destroy()

    def on_plugin_prefs__clicked(self, button):
        plugin_info = self._get_selected_plugin()
        if plugin_info:
            self.plugin_manager.show_plugin_preferences(plugin_info.name,
                                                        self.dialog)

    def _get_selected_plugin(self):
        plugin_list = self.ob.get_widget('plugins_list')
        selection = plugin_list.get_selection()
        model, model_iter = selection.get_selected()
        if model_iter:
            return model.get_value(model_iter, COL_PLUGININFO)



from decimal import Decimal
from django.core import urlresolvers
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils.http import urlencode
from django.utils.translation import ugettext as _
from django.views.decorators.cache import never_cache
from livesettings import config_get_group, config_value 
from payment.config import gateway_live
from payment.utils import get_processor_by_key
from payment.views import payship
from satchmo_store.shop.models import Cart
from satchmo_store.shop.models import Order, OrderPayment
from satchmo_utils.dynamic import lookup_url, lookup_template
from sys import exc_info
from traceback import format_exception
import logging
import urllib2

log = logging.getLogger()

def pay_ship_info(request):
    return payship.base_pay_ship_info(request,
        config_get_group('PAYMENT_PAYPAL'), payship.simple_pay_ship_process_form,
        'shop/checkout/paypal/pay_ship.html')
pay_ship_info = never_cache(pay_ship_info)


def confirm_info(request):
    payment_module = config_get_group('PAYMENT_PAYPAL')

    try:
        order = Order.objects.from_request(request)
    except Order.DoesNotExist:
        url = lookup_url(payment_module, 'satchmo_checkout-step1')
        return HttpResponseRedirect(url)

    tempCart = Cart.objects.from_request(request)
    if tempCart.numItems == 0 and not order.is_partially_paid:
        template = lookup_template(payment_module, 'shop/checkout/empty_cart.html')
        return render_to_response(template,
                                  context_instance=RequestContext(request))

    # Check if the order is still valid
    if not order.validate(request):
        context = RequestContext(request,
                                 {'message': _('Your order is no longer valid.')})
        return render_to_response('shop/404.html', context_instance=context)

    template = lookup_template(payment_module, 'shop/checkout/paypal/confirm.html')
    if payment_module.LIVE.value:
        log.debug("live order on %s", payment_module.KEY.value)
        url = payment_module.POST_URL.value
        account = payment_module.BUSINESS.value
    else:
        url = payment_module.POST_TEST_URL.value
        account = payment_module.BUSINESS_TEST.value

    try:
        address = lookup_url(payment_module,
            payment_module.RETURN_ADDRESS.value, include_server=True)
    except urlresolvers.NoReverseMatch:
        address = payment_module.RETURN_ADDRESS.value
    
    processor_module = payment_module.MODULE.load_module('processor')
    processor = processor_module.PaymentProcessor(payment_module)
    processor.create_pending_payment(order=order)
    default_view_tax = config_value('TAX', 'DEFAULT_VIEW_TAX') 
  
    recurring = None
    order_items = order.orderitem_set.all()
    for item in order_items:
        if item.product.is_subscription:
            recurring = {'product':item.product, 'price':item.product.price_set.all()[0].price.quantize(Decimal('.01')),}
            trial0 = recurring['product'].subscriptionproduct.get_trial_terms(0)
            if len(order_items) > 1 or trial0 is not None or recurring['price'] < order.balance:
                recurring['trial1'] = {'price': order.balance,}
                if trial0 is not None:
                    recurring['trial1']['expire_length'] = trial0.expire_length
                    recurring['trial1']['expire_unit'] = trial0.expire_unit[0]
                # else:
                #     recurring['trial1']['expire_length'] = recurring['product'].subscriptionproduct.get_trial_terms(0).expire_length
                trial1 = recurring['product'].subscriptionproduct.get_trial_terms(1)
                if trial1 is not None:
                    recurring['trial2']['expire_length'] = trial1.expire_length
                    recurring['trial2']['expire_unit'] = trial1.expire_unit[0]
                    recurring['trial2']['price'] = trial1.price
 
    ctx = RequestContext(request, {'order': order,
     'post_url': url,
     'default_view_tax': default_view_tax, 
     'business': account,
     'currency_code': payment_module.CURRENCY_CODE.value,
     'return_address': address,
     'invoice': order.id,
     'subscription': recurring,
     'PAYMENT_LIVE' : gateway_live(payment_module)
    })

    return render_to_response(template, context_instance=ctx)
confirm_info = never_cache(confirm_info)

def ipn(request):
    """PayPal IPN (Instant Payment Notification)
    Cornfirms that payment has been completed and marks invoice as paid.
    Adapted from IPN cgi script provided at http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/456361"""
    payment_module = config_get_group('PAYMENT_PAYPAL')
    if payment_module.LIVE.value:
        log.debug("Live IPN on %s", payment_module.KEY.value)
        url = payment_module.POST_URL.value
        account = payment_module.BUSINESS.value
    else:
        log.debug("Test IPN on %s", payment_module.KEY.value)
        url = payment_module.POST_TEST_URL.value
        account = payment_module.BUSINESS_TEST.value
    PP_URL = url

    try:
        data = request.POST
        log.debug("PayPal IPN data: " + repr(data))
        if not confirm_ipn_data(data, PP_URL):
            return HttpResponse()

        if not 'payment_status' in data or not data['payment_status'] == "Completed":
            # We want to respond to anything that isn't a payment - but we won't insert into our database.
             log.info("Ignoring IPN data for non-completed payment.")
             return HttpResponse()

        try:
            invoice = data['invoice']
        except:
            invoice = data['item_number']

        gross = data['mc_gross']
        txn_id = data['txn_id']

        if not OrderPayment.objects.filter(transaction_id=txn_id).count():
            # If the payment hasn't already been processed:
            order = Order.objects.get(pk=invoice)
            
            order.add_status(status='New', notes=_("Paid through PayPal."))
            processor = get_processor_by_key('PAYMENT_PAYPAL')
            payment = processor.record_payment(order=order, amount=gross, transaction_id=txn_id)
            
            if 'memo' in data:
                if order.notes:
                    notes = order.notes + "\n"
                else:
                    notes = ""
                
                order.notes = notes + _('---Comment via Paypal IPN---') + u'\n' + data['memo']
                order.save()
                log.debug("Saved order notes from Paypal")
            
            for item in order.orderitem_set.filter(product__subscriptionproduct__recurring=True, completed=False):
                item.completed = True
                item.save()
            for cart in Cart.objects.filter(customer=order.contact):
                cart.empty()
                
    except:
        log.exception(''.join(format_exception(*exc_info())))

    return HttpResponse()

def confirm_ipn_data(data, PP_URL):
    # data is the form data that was submitted to the IPN URL.

    newparams = {}
    for key in data.keys():
        newparams[key] = data[key]

    newparams['cmd'] = "_notify-validate"
    params = urlencode(newparams)

    req = urllib2.Request(PP_URL)
    req.add_header("Content-type", "application/x-www-form-urlencoded")
    fo = urllib2.urlopen(req, params)

    ret = fo.read()
    if ret == "VERIFIED":
        log.info("PayPal IPN data verification was successful.")
    else:
        log.info("PayPal IPN data verification failed.")
        log.debug("HTTP code %s, response text: '%s'" % (fo.code, ret))
        return False

    return True



#!/usr/bin/env python
# Copyright (C) 2007-2013 Etienne Robillard <erob@gthcfoundation.org>
# All rights reserved
# <LICENSE=APACHEV2>
"""Common HTTP utilities."""

from .httpserver import HTTPServer, get_bind_addr, daemonize
from .decorators import post_required

#from datetime import datetime
#def last_modified(timestamp):
#    "Returns the Last-Modified string since the epoc (see time.time)"
#    return str(datetime.fromtimestamp(timestamp))




import django.dispatch

#
# Signals sent by Orders
#

#: Sent when an order is about to be cancelled and asks listeners if they allow
#: to do so.
#:
#: By default, orders in states 'Shipped', 'Completed' and 'Cancelled' are not
#: allowed to be cancelled. The default verdict is stored in
#: ``order.is_cancellable`` flag. Listeners can modify this flag, according to
#: their needs.
#:
#: :param sender: The order about to be cancelled.
#: :type sender: ``satchmo_store.shop.models.Order``
#:
#: :param order: The order about to be cancelled.
#: :type order: ``satchmo_store.shop.models.Order``
#:
#: .. Note:: *order* argument is the same as *sender*.
order_cancel_query = django.dispatch.Signal()

#: Sent when an order has been cancelled; it's status already reflects it and
#: has been saved to the database (e.g. payment gateway cancels payment).
#:
#: :param sender: The order that was cancelled.
#: :type sender: ``satchmo_store.shop.models.Order``
#:
#: :param order: The order that was cancelled.
#: :type order: ``satchmo_store.shop.models.Order``
#:
#: .. Note:: *order* argument is the same as *sender*.
order_cancelled = django.dispatch.Signal()

#: Sent when an order is complete and the balance goes to zero during a save.
#:
#: :param sender: The order that was successful.
#: :type sender: ``satchmo_store.shop.models.Order``
#: :param order: The order that was successful.
#: :type order: ``satchmo_store.shop.models.Order``
#:
#: .. Note:: *order* argument is the same as *sender*.
order_success = django.dispatch.Signal()

#: Sent by the order when its status has changed.
#satchmo_order_status_changed.send(self.order, oldstatus=oldstatus, newstatus=status, order=order)
satchmo_order_status_changed=django.dispatch.Signal()

#
# Signals sent by the Cart system
#

#: Sent by 'views.smart_add` to allow listeners to optionally change the
#: responding function.
#:
#: :param sender: The cart model
#: :type sender: ``satchmo_store.shop.models.Cart``
#:
#: :param request: The request used by the view
#: :type request: ``django.http.HttpRequest``
#:
#: :param method: A dictionary containing a single key ``view`` to be updated
#:   with the function to be called by smart_add. For example::
#:
#:       method = {'view': cart.add }
#:
#: .. Note:: *sender* is not a class instance.
cart_add_view = django.dispatch.Signal()

#: Sent after an item has been successfully added to the cart.
#:
#: :param sender: The cart the cart item was added to.
#: :type sender: ``satchmo_store.shop.models.Cart``
#:
#: :param cart: The cart the cart item was added to.
#: :type cart: ``satchmo_store.shop.models.Cart``
#:
#: :param cartitem: The cart item that was added to the cart.
#: :type cartitem: ``satchmo_store.shop.models.CartItem``
#:
#: :param product: The product that was added to the cart.
#: :type product: ``product.models.Product``
#:
#: :param form: The POST data for the form used to add the item to the cart.
#:
#: :param request: The request that used in the view to add the item to the
#:   cart.
#: :type request: ``django.http.HttpRequest``
#:
#: .. Note:: *cart* is the same as *sender*.
satchmo_cart_add_complete=django.dispatch.Signal()

#: Sent before an item is added to the cart.
#:
#: :param sender: The cart the cart item is being added to.
#: :type sender: ``satchmo_store.shop.models.Cart``
#:
#: :param cart: The cart the cart item is being added to.
#: :type cart: ``satchmo_store.shop.models.Cart``
#:
#: :param cartitem: The cart item that is being added to the cart.
#: :type cartitem: ``satchmo_store.shop.models.CartItem``
#:
#: :param added_quantity: The number of ``satchmo_store.shop.models.CartItem``
#:   instances items being added to the cart.
#: :type added_quantity: ``decimal.Decimal``
#:
#: :param details: A list of dictionaries containing additional details about the
#:   item if the item is a custom product or a gift certificate product. Each
#:   dictionary has the following entries:
#:
#:   :name: The name of the detail
#:   :value: The value of the detail
#:   :value: The value of the detail
#:   :sort_order: The order the detail should be listed in displays
#:   :price_change: The price change of the detail
#:
#:     :default: zero
#:
#: .. Note:: *cart* is the same as *sender*.
satchmo_cart_add_verify=django.dispatch.Signal()

#: Sent whenever the status of the cart has changed. For example, when an item
#: is added, removed, or had it's quantity updated.
#:
#: :param sender: The cart that was changed.
#: :type sender: ``satchmo_store.shop.models.Cart``
#:
#: :param cart: The cart that was changed.
#: :type cart: ``satchmo_store.shop.models.Cart``
#:
#: :param request: The request that used in the view to add the item to the
#:   cart.
#: :type request: ``django.http.HttpRequest``
#:
#: .. Note:: *cart* is the same as *sender*.
satchmo_cart_changed=django.dispatch.Signal()

#: Sent by the pricing system to allow price overrides when displaying line item
#: prices.
#:
#: :param sender: The cart item being queried for price overrides.
#: :type sender: ``satchmo_store.shop.models.CartItem``
#:
#: :param cartitem: The cart item being queried for price overrides.
#: :type cartitem: ``satchmo_store.shop.models.CartItem``
#:
#: .. Note:: *cartitem* is the same as *sender*.
satchmo_cartitem_price_query=django.dispatch.Signal()

#: Sent before an item is added to the cart so that listeners can update product
#: details.
#:
#: :param sender: The cart the cart item is being added to.
#: :type sender: ``satchmo_store.shop.models.Cart``
#:
#: :param product: The product that is being added to the cart.
#: :type product: ``product.models.Product`` or
#:   ``product.models.ConfigurableProduct``
#:
#: :param quantity: The number of ``satchmo_store.shop.models.CartItem``
#:    instances items being added to the cart.
#: :type quantity: ``decimal.Decimal``
#:
#: :param details: A list of dictionaries containing additional details about the
#:   item if the item is a custom product or a gift certificate product. Each
#:   dictionary has the following entries:
#:
#:   :name: The name of the detail
#:   :value: The value of the detail
#:   :value: The value of the detail
#:   :sort_order: The order the detail should be listed in displays
#:   :price_change: The price change of the detail
#:
#: :param form: The POST data for the form used to add the item to the cart.
#:
#: :param request: The request that used in the view to add the item to the
#:   cart.
#: :type request: ``django.http.HttpRequest``
#:
#: .. Note:: *cart* is the same as *sender*.
satchmo_cart_details_query=django.dispatch.Signal()

#
# Miscellaneous Signals
#

#: Sent when ``satchmo_store.shop.context_processors.settings()`` is invoked,
#: before the context is returned. This signal can be used to modify the context
#: returned by the context processor.
#:
#: :param sender: The current store configuration
#: :type sender: ``satchmo_store.shop.models.Config``
#:
#: :param context: A dictionary containing the context to be returned by the
#:   context processor. The dictionary contains:
#:
#:   :shop_base: The base URL for the store
#:   :shop: An instance of ``satchmo_store.shop.models.Config`` representing the
#:     current store configuration
#:   :shop_name: The shop name
#:   :media_url: The current media url, taking into account SSL
#:   :cart_count: The number of items in the cart
#:   :cart: An instance of ``satchmo_store.shop.models.Cart`` representing the
#:     current cart
#:   :categories: A ``QuerySet`` of all the ``product.models.Category`` objects
#:     for the current site.
#:   :is_secure: A boolean representing weather or not SSL is enabled
#:   :request: The ``HttpRequest`` object passed into the context processor
#:   :login_url: The login url defined in ``settings.LOGIN_URL``
#:   :logout_url: The logout url defined in ``settings.LOGOUT_URL``
#:   :sale: An instance of ``product.models.Discount`` if there is a current
#:     sale, or ``None``
satchmo_context = django.dispatch.Signal()

#: Sent after each item from the cart is copied into an order.
#:
#: :param sender: The cart the cart items are being copied into.
#: :type sender: ``satchmo_store.shop.models.Cart``
#:
#: :param cartitem: The cart item being copied into an order.
#: :type cartitem: ``satchmo_store.shop.models.CartItem``
#:
#: :param order: The order having items copied into it.
#: :type order: ``satchmo_store.shop.models.Order``
#:
#: :param orderitem: The order item being added to the order.
#: :type orderitem: ``satchmo_store.shop.models.OrderItem``
satchmo_post_copy_item_to_order=django.dispatch.Signal()

#: Sent by the order during the calculation of the total.
#satchmo_shipping_price_query.send(order, adjustment=shipadjust)
satchmo_shipping_price_query = django.dispatch.Signal()

#: Sent to determine where to redirect a user for a ``DownloadableProduct``.
#:
#: :param sender: ``None``
#: :type sender: ``None``
#:
#: :param file: The ``'file'`` field of the ``DownloadableProduct``.
#: :type file: ``django.db.models.fields.files.FileField``
#:
#: :param product: The product which is being downloaded.
#: :type product: ``product.models.DownloadableProduct``
#:
#: :param url_dict: A dictionary containing a single entry, ``'url'``, the URL
#:   which the user will be redirected to. Listeners should modify this value
#:   to change the redirect URL.
#:
#: .. Warning::
#:    For a sane ``filename`` parameter in the ``Content-Disposition`` header,
#:    users are cautioned against appending a trailing slash(``'/'``) to the
#:    URL.
sendfile_url_for_file = django.dispatch.Signal()

#
# Signals sent by email system
#

#: Sent by ``satchmo_store.mail.send_store_mail()`` before the message body is
#: rendered.
#:
#: Takes the same arguments as :data:`sending_store_mail`.
#:
#: .. Note::
#:   :ref:`send_mail_args <send_mail_args>` does not contain the ``'subject'``
#:   entry.
#:
#: .. Note::
#:   If the ``'message'`` entry is set in :ref:`send_mail_args <send_mail_args>`
#:   by a listener, it will be used instead of the rendered result in
#:   ``send_store_mail()``.
rendering_store_mail = django.dispatch.Signal()

#: Sent by ``satchmo_store.mail.send_store_mail()`` just before ``send_mail()``
#: is invoked.
#:
#: Listeners may raise ``satchmo_store.mail.ShouldNotSendMail``.
#:
#: If they choose to invoke ``django.mail.EmailMessage.send()``, any errors
#: raised will be handled by ``send_store_mail()``; they should consequently
#: raise ``ShouldNotSendMail`` to avoid re-sending the email.
#:
#: :param sender: Defaults to ``None``, unless the sender argument to
#:   ``send_store_mail()`` is specified; see below.
#:
#: .. _send_mail_args:
#:
#: :param send_mail_args: A dictionary containing the keyword arguments passed
#:   to ``send_mail()``:
#:
#:   - subject
#:   - message
#:   - from_email
#:   - recipient_list
#:   - fail_silently
#:
#: :param context: The context used to render the message body; by default, it
#:   contains the 'shop_name' key, but may contain other keys, depending on the
#:   `context` argument to ``send_store_mail()``.
#:
#: :param `**kwargs`: Additional keyword arguments received by
#:   ``send_store_mail()``.
#:
#: .. Note::
#:
#:    If the *context* argument to ``send_store_mail()`` contains the entry
#:    `send_mail_args`, it will not be available in the listener's *context*
#:    dictionary.
#:
#: Example::
#:
#:   from satchmo_store.shop.signals import order_notice_sender
#:
#:   def modify_subject(sender, send_mail_args={}, context={}, **kwargs):
#:     if not ('shop_name' in context and 'order' in context):
#:       return
#:
#:     send_mail_args['subject'] = '[%s] Woohoo! You got a *new* order! (ID: #%d)' % \
#:         (context['shop_name'], context['order'].id)
#:
#:   sending_store_mail.connect(modify_subject, sender=order_notice_sender)
#:
sending_store_mail = django.dispatch.Signal()

#
# Email senders.
#

# satchmo_store.notification
order_confirmation_sender = object()
order_notice_sender = object()
ship_notice_sender = object()

# satchmo_store.shop.views.contact
contact_sender = object()

# satchmo_store.registration
registration_sender = object()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2007-2018 Etienne Robillard <tkadm30@yandex.com>
# All rights reserved.
#
# This file is part of the django-hotsauce distribution.
# Please see the "LICENSE" file for details.

import sys
import os
import argparse

# Hack for adding lib/sandbox to the pythonpath
currentdir = os.getcwd()

sys.path.append(os.path.join(currentdir, 'lib')) # test apps
sys.path.append(os.path.join(currentdir, 'lib/site-packages')) # contrib apps

if not 'django' in sys.modules.keys():
    import django
    os.environ['DJANGO_HOME'] = django.__path__[0]
    os.environ['DJANGO_SETTINGS_MODULE'] = 'local_settings'
    sys.path.append(os.environ['DJANGO_HOME'])


# examples and contrib apps
vpath = (
    os.path.join(currentdir, '../examples/lib'),
    #os.path.join(currentdir, '../examples/contrib'),
    os.path.join(currentdir, '../examples/lib/site-packages')
    ) 
for path in vpath:
    sys.path.append(path)

from notmm.utils.testlib import SimpleTestRunner
from notmm.utils.configparse import loadconf

# TODO: Add "-c config.ini" support using getopt
testdir = currentdir
app_conf = loadconf(os.path.join(testdir, 'development.ini'))

def parse_command_line(argv=sys.argv):
    parser = argparse.ArgumentParser(__file__)
    parser.add_argument('-v', '--verbose', \
        dest="verbose", \
        action="store_true", \
        help="Enable maintainer mode", \
        default=False
        )
    # $DJANGO_CONF 
    parser.add_argument('-c', '--config', \
        dest="app_conf", \
        nargs=1, \
        help="Path to a alternative development.ini (Default: $DJANGO_CONF)",
        default=os.path.join(currentdir, "development.ini")
        )
    # $DJANGO_SETTINGS_MODULE    
    parser.add_argument('--settings', \
        dest="settings", \
        nargs=1, \
        required=False, \
        help="Django settings module (Default: $DJANGO_SETTINGS_MODULE)", \
        default=os.environ['DJANGO_SETTINGS_MODULE']
        )
    # Adds support for $PYTHONPATH overriding
    parser.add_argument('--pythonpath', \
        dest="vpath",   \
        default=vpath, \
        required=False, 
        )
    parser.add_argument('-C', '--collections', 
        dest="collections", \
        type=str, \
        required=False, \
        default=app_conf['testrunner']['collections'],
        help="List of subpackages to search for test scripts"
        )
    
    #interactive option here

    options = parser.parse_args(args=argv[1:])
    return options

def main(verbosity=1):
    options = parse_command_line()
    if options.verbose:
        verbosity = 2
    runner = SimpleTestRunner(options, verbosity=verbosity)
    runner.run()

if __name__ == '__main__':
   main()





from django.conf.urls.defaults import *
from django.db import models
from livesettings import config_get_group, config_get, config_value
from livesettings.models import SettingNotSet
import logging

log = logging.getLogger('payment.urls')

config = config_get_group('PAYMENT')

urlpatterns = patterns('payment.views',
     (r'^$', 'contact.contact_info_view', {'SSL': config.SSL.value}, 'satchmo_checkout-step1'),
     (r'^success/$', 'checkout.success', {'SSL' : config.SSL.value}, 'satchmo_checkout-success'),
     (r'custom/charge/(?P<orderitem_id>\d+)/$', 'balance.charge_remaining', {}, 'satchmo_charge_remaining'),
     (r'custom/charge/$', 'balance.charge_remaining_post', {}, 'satchmo_charge_remaining_post'),
     (r'^balance/(?P<order_id>\d+)/$', 'balance.balance_remaining_order', {'SSL' : config.SSL.value}, 'satchmo_balance_remaining_order'),
     (r'^balance/$', 'balance.balance_remaining', {'SSL' : config.SSL.value}, 'satchmo_balance_remaining'),
     (r'^cron/$', 'cron.cron_rebill', {}, 'satchmo_cron_rebill'),
     (r'^mustlogin/$', 'contact.authentication_required', {'SSL' : config.SSL.value}, 'satchmo_checkout_auth_required'),
)

# now add all enabled module payment settings

def make_urlpatterns():
    patterns = []
    for app in models.get_apps():
        if hasattr(app, 'PAYMENT_PROCESSOR'):
            parts = app.__name__.split('.')
            key = parts[-2].upper()
            modulename = 'PAYMENT_%s' % key
            name = app.__name__
            name = name[:name.rfind('.')]
            #log.debug('payment module=%s, key=%s', modulename, key)
            # BJK: commenting out Bursar settings here
            # try:
            #     cfg = config_get(modulename, 'INTERFACE_MODULE')
            #     interface = cfg.editor_value
            # except SettingNotSet:
            #     interface = name[:name.rfind('.')]
            # urlmodule = "%s.urls" % interface
            urlmodule = '.'.join(parts[:-1]) + '.urls'
            urlbase = config_value(modulename, 'URL_BASE')
            log.debug('Found payment processor: %s, adding urls at %s', key, urlbase)
            patterns.append(url(urlbase, [urlmodule, '', '']))
    return tuple(patterns)
    
urlpatterns += make_urlpatterns()



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):

        # Adding M2M table for field valid_products on 'Discount'
        db.create_table('product_discount_valid_products', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('discount', models.ForeignKey(orm['product.discount'], null=False)),
            ('product', models.ForeignKey(orm['product.product'], null=False))
        ))
        db.create_unique('product_discount_valid_products', ['discount_id', 'product_id'])

        if not db.dry_run:
            for discount in orm['product.discount'].objects.all():
                for product in discount.validProducts.all():
                    discount.valid_products.add(product)
                discount.save()


    def backwards(self, orm):

        # Removing M2M table for field valid_products on 'Discount'
        db.delete_table('product_discount_valid_products')


    models = {
        'product.attributeoption': {
            'Meta': {'object_name': 'AttributeOption'},
            'description': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'error_message': ('django.db.models.fields.CharField', [], {'default': "u'Invalid Entry'", 'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.SlugField', [], {'max_length': '100', 'db_index': 'True'}),
            'sort_order': ('django.db.models.fields.IntegerField', [], {'default': '1'}),
            'validation': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'product.category': {
            'Meta': {'unique_together': "(('site', 'slug'),)", 'object_name': 'Category'},
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'meta': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'ordering': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'child'", 'blank': 'True', 'null': 'True', 'to': "orm['product.Category']"}),
            'related_categories': ('django.db.models.fields.related.ManyToManyField', [], {'related_name': "'related_categories'", 'blank': 'True', 'null': 'True', 'to': "orm['product.Category']"}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['sites.Site']"}),
            'slug': ('django.db.models.fields.SlugField', [], {'db_index': 'True', 'max_length': '50', 'blank': 'True'})
        },
        'product.categoryimage': {
            'Meta': {'unique_together': "(('category', 'sort'),)", 'object_name': 'CategoryImage'},
            'caption': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'category': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'images'", 'blank': 'True', 'null': 'True', 'to': "orm['product.Category']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'picture': ('satchmo_utils.thumbnail.field.ImageWithThumbnailField', [], {'name_field': "'_filename'", 'max_length': '200'}),
            'sort': ('django.db.models.fields.IntegerField', [], {'default': '0'})
        },
        'product.categoryimagetranslation': {
            'Meta': {'unique_together': "(('categoryimage', 'languagecode', 'version'),)", 'object_name': 'CategoryImageTranslation'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'caption': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'categoryimage': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'to': "orm['product.CategoryImage']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'languagecode': ('django.db.models.fields.CharField', [], {'max_length': '10'}),
            'version': ('django.db.models.fields.IntegerField', [], {'default': '1'})
        },
        'product.categorytranslation': {
            'Meta': {'unique_together': "(('category', 'languagecode', 'version'),)", 'object_name': 'CategoryTranslation'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'category': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'to': "orm['product.Category']"}),
            'description': ('django.db.models.fields.TextField', [], {'default': "''", 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'languagecode': ('django.db.models.fields.CharField', [], {'max_length': '10'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'version': ('django.db.models.fields.IntegerField', [], {'default': '1'})
        },
        'product.configurableproduct': {
            'Meta': {'object_name': 'ConfigurableProduct'},
            'create_subs': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'option_group': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['product.OptionGroup']", 'blank': 'True'}),
            'product': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['product.Product']", 'unique': 'True', 'primary_key': 'True'})
        },
        'product.customproduct': {
            'Meta': {'object_name': 'CustomProduct'},
            'deferred_shipping': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'downpayment': ('django.db.models.fields.IntegerField', [], {'default': '20'}),
            'option_group': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['product.OptionGroup']", 'blank': 'True'}),
            'product': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['product.Product']", 'unique': 'True', 'primary_key': 'True'})
        },
        'product.customtextfield': {
            'Meta': {'unique_together': "(('slug', 'products'),)", 'object_name': 'CustomTextField'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '40'}),
            'price_change': ('satchmo_utils.fields.CurrencyField', [], {'null': 'True', 'max_digits': '14', 'decimal_places': '6', 'blank': 'True'}),
            'products': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'custom_text_fields'", 'to': "orm['product.CustomProduct']"}),
            'slug': ('django.db.models.fields.SlugField', [], {'db_index': 'True', 'max_length': '50', 'blank': 'True'}),
            'sort_order': ('django.db.models.fields.IntegerField', [], {'default': '0'})
        },
        'product.customtextfieldtranslation': {
            'Meta': {'unique_together': "(('customtextfield', 'languagecode', 'version'),)", 'object_name': 'CustomTextFieldTranslation'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'customtextfield': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'to': "orm['product.CustomTextField']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'languagecode': ('django.db.models.fields.CharField', [], {'max_length': '10'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'version': ('django.db.models.fields.IntegerField', [], {'default': '1'})
        },
        'product.discount': {
            'Meta': {'object_name': 'Discount'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'allValid': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'allowedUses': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'amount': ('satchmo_utils.fields.CurrencyField', [], {'null': 'True', 'max_digits': '8', 'decimal_places': '2', 'blank': 'True'}),
            'automatic': ('django.db.models.fields.NullBooleanField', [], {'default': 'False', 'null': 'True', 'blank': 'True'}),
            'code': ('django.db.models.fields.CharField', [], {'max_length': '20', 'unique': 'True'}),
            'description': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'endDate': ('django.db.models.fields.DateField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'minOrder': ('satchmo_utils.fields.CurrencyField', [], {'null': 'True', 'max_digits': '8', 'decimal_places': '2', 'blank': 'True'}),
            'numUses': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'percentage': ('django.db.models.fields.DecimalField', [], {'null': 'True', 'max_digits': '5', 'decimal_places': '2', 'blank': 'True'}),
            'shipping': ('django.db.models.fields.CharField', [], {'default': "'NONE'", 'max_length': '10', 'null': 'True', 'blank': 'True'}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['sites.Site']"}),
            'startDate': ('django.db.models.fields.DateField', [], {}),
            'validProducts': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['product.Product']", 'null': 'True', 'blank': 'True'}),
            'valid_products': ('django.db.models.fields.related.ManyToManyField', [], {'related_name': "'discounts_temp'", 'blank': 'True', 'null': 'True', 'to': "orm['product.Product']"})
        },
        'product.downloadableproduct': {
            'Meta': {'object_name': 'DownloadableProduct'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'expire_minutes': ('django.db.models.fields.IntegerField', [], {}),
            'file': ('django.db.models.fields.files.FileField', [], {'max_length': '100'}),
            'num_allowed_downloads': ('django.db.models.fields.IntegerField', [], {}),
            'product': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['product.Product']", 'unique': 'True', 'primary_key': 'True'})
        },
        'product.option': {
            'Meta': {'unique_together': "(('option_group', 'value'),)", 'object_name': 'Option'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'option_group': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['product.OptionGroup']"}),
            'price_change': ('satchmo_utils.fields.CurrencyField', [], {'null': 'True', 'max_digits': '14', 'decimal_places': '6', 'blank': 'True'}),
            'sort_order': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'product.optiongroup': {
            'Meta': {'object_name': 'OptionGroup'},
            'description': ('django.db.models.fields.CharField', [], {'max_length': '100', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['sites.Site']"}),
            'sort_order': ('django.db.models.fields.IntegerField', [], {'default': '0'})
        },
        'product.optiongrouptranslation': {
            'Meta': {'unique_together': "(('optiongroup', 'languagecode', 'version'),)", 'object_name': 'OptionGroupTranslation'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'default': "''", 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'languagecode': ('django.db.models.fields.CharField', [], {'max_length': '10'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'optiongroup': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'to': "orm['product.OptionGroup']"}),
            'version': ('django.db.models.fields.IntegerField', [], {'default': '1'})
        },
        'product.optiontranslation': {
            'Meta': {'unique_together': "(('option', 'languagecode', 'version'),)", 'object_name': 'OptionTranslation'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'languagecode': ('django.db.models.fields.CharField', [], {'max_length': '10'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'option': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'to': "orm['product.Option']"}),
            'version': ('django.db.models.fields.IntegerField', [], {'default': '1'})
        },
        'product.price': {
            'Meta': {'unique_together': "(('product', 'quantity', 'expires'),)", 'object_name': 'Price'},
            'expires': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'price': ('satchmo_utils.fields.CurrencyField', [], {'max_digits': '14', 'decimal_places': '6'}),
            'product': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['product.Product']"}),
            'quantity': ('django.db.models.fields.DecimalField', [], {'default': "'1.0'", 'max_digits': '18', 'decimal_places': '6'})
        },
        'product.product': {
            'Meta': {'unique_together': "(('site', 'sku'), ('site', 'slug'))", 'object_name': 'Product'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'also_purchased': ('django.db.models.fields.related.ManyToManyField', [], {'related_name': "'also_products'", 'blank': 'True', 'null': 'True', 'to': "orm['product.Product']"}),
            'category': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['product.Category']", 'blank': 'True'}),
            'date_added': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'default': "''", 'blank': 'True'}),
            'featured': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'height': ('django.db.models.fields.DecimalField', [], {'null': 'True', 'max_digits': '6', 'decimal_places': '2', 'blank': 'True'}),
            'height_units': ('django.db.models.fields.CharField', [], {'max_length': '3', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'items_in_stock': ('django.db.models.fields.DecimalField', [], {'default': "'0'", 'max_digits': '18', 'decimal_places': '6'}),
            'length': ('django.db.models.fields.DecimalField', [], {'null': 'True', 'max_digits': '6', 'decimal_places': '2', 'blank': 'True'}),
            'length_units': ('django.db.models.fields.CharField', [], {'max_length': '3', 'null': 'True', 'blank': 'True'}),
            'meta': ('django.db.models.fields.TextField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'ordering': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'related_items': ('django.db.models.fields.related.ManyToManyField', [], {'related_name': "'related_products'", 'blank': 'True', 'null': 'True', 'to': "orm['product.Product']"}),
            'shipclass': ('django.db.models.fields.CharField', [], {'default': "'DEFAULT'", 'max_length': '10'}),
            'short_description': ('django.db.models.fields.TextField', [], {'default': "''", 'max_length': '200', 'blank': 'True'}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['sites.Site']"}),
            'sku': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'db_index': 'True', 'max_length': '255', 'blank': 'True'}),
            'taxClass': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['product.TaxClass']", 'null': 'True', 'blank': 'True'}),
            'taxable': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'total_sold': ('django.db.models.fields.DecimalField', [], {'default': "'0'", 'max_digits': '18', 'decimal_places': '6'}),
            'weight': ('django.db.models.fields.DecimalField', [], {'null': 'True', 'max_digits': '8', 'decimal_places': '2', 'blank': 'True'}),
            'weight_units': ('django.db.models.fields.CharField', [], {'max_length': '3', 'null': 'True', 'blank': 'True'}),
            'width': ('django.db.models.fields.DecimalField', [], {'null': 'True', 'max_digits': '6', 'decimal_places': '2', 'blank': 'True'}),
            'width_units': ('django.db.models.fields.CharField', [], {'max_length': '3', 'null': 'True', 'blank': 'True'})
        },
        'product.productattribute': {
            'Meta': {'object_name': 'ProductAttribute'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'languagecode': ('django.db.models.fields.CharField', [], {'max_length': '10', 'null': 'True', 'blank': 'True'}),
            'option': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['product.AttributeOption']"}),
            'product': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['product.Product']"}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        'product.productimage': {
            'Meta': {'object_name': 'ProductImage'},
            'caption': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'picture': ('satchmo_utils.thumbnail.field.ImageWithThumbnailField', [], {'name_field': "'_filename'", 'max_length': '200'}),
            'product': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['product.Product']", 'null': 'True', 'blank': 'True'}),
            'sort': ('django.db.models.fields.IntegerField', [], {'default': '0'})
        },
        'product.productimagetranslation': {
            'Meta': {'unique_together': "(('productimage', 'languagecode', 'version'),)", 'object_name': 'ProductImageTranslation'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'caption': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'languagecode': ('django.db.models.fields.CharField', [], {'max_length': '10'}),
            'productimage': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'to': "orm['product.ProductImage']"}),
            'version': ('django.db.models.fields.IntegerField', [], {'default': '1'})
        },
        'product.productpricelookup': {
            'Meta': {'object_name': 'ProductPriceLookup'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'discountable': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'items_in_stock': ('django.db.models.fields.DecimalField', [], {'max_digits': '18', 'decimal_places': '6'}),
            'key': ('django.db.models.fields.CharField', [], {'max_length': '60', 'null': 'True'}),
            'parentid': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'price': ('django.db.models.fields.DecimalField', [], {'max_digits': '14', 'decimal_places': '6'}),
            'productslug': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'quantity': ('django.db.models.fields.DecimalField', [], {'max_digits': '18', 'decimal_places': '6'}),
            'siteid': ('django.db.models.fields.IntegerField', [], {})
        },
        'product.producttranslation': {
            'Meta': {'unique_together': "(('product', 'languagecode', 'version'),)", 'object_name': 'ProductTranslation'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'default': "''", 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'languagecode': ('django.db.models.fields.CharField', [], {'max_length': '10'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'product': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'to': "orm['product.Product']"}),
            'short_description': ('django.db.models.fields.TextField', [], {'default': "''", 'max_length': '200', 'blank': 'True'}),
            'version': ('django.db.models.fields.IntegerField', [], {'default': '1'})
        },
        'product.productvariation': {
            'Meta': {'object_name': 'ProductVariation'},
            'options': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['product.Option']"}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['product.ConfigurableProduct']"}),
            'product': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['product.Product']", 'unique': 'True', 'primary_key': 'True'})
        },
        'product.subscriptionproduct': {
            'Meta': {'object_name': 'SubscriptionProduct'},
            'expire_length': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'expire_unit': ('django.db.models.fields.CharField', [], {'default': "'DAY'", 'max_length': '5'}),
            'is_shippable': ('django.db.models.fields.IntegerField', [], {'max_length': '1'}),
            'product': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['product.Product']", 'unique': 'True', 'primary_key': 'True'}),
            'recurring': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'recurring_times': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'})
        },
        'product.taxclass': {
            'Meta': {'object_name': 'TaxClass'},
            'description': ('django.db.models.fields.CharField', [], {'max_length': '30'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '20'})
        },
        'product.trial': {
            'Meta': {'object_name': 'Trial'},
            'expire_length': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'price': ('satchmo_utils.fields.CurrencyField', [], {'null': 'True', 'max_digits': '10', 'decimal_places': '2'}),
            'subscription': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['product.SubscriptionProduct']"})
        },
        'sites.site': {
            'Meta': {'object_name': 'Site', 'db_table': "'django_site'"},
            'domain': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        }
    }

    complete_apps = ['product']



import os
import logging
#import django
#django.setup()
from django.conf import settings

VERSION = "1.0"

def configureLogging(level='debug'):
    logging_level_choices = {
        'error' : logging.ERROR,
        'debug' : logging.DEBUG,
        'info'  : logging.INFO,
        'critical'  : logging.CRITICAL,
    }
    loglevel = logging_level_choices[level]
    logfile = os.path.join(settings.SATCHMO_LOGDIR, 'satchmo.log')

    logging.basicConfig(
        level=logging.DEBUG,
        format='%(asctime)s %(name)-12s %(levelname)-8s %(message)s',
        datefmt='%a, %d %b %Y %H:%M:%S',
    filename=logfile,
    filemode='w')

    logging.getLogger('keyedcache').setLevel(loglevel)
    logging.getLogger('l10n').setLevel(loglevel)
    
    logging.info("Satchmo Logging subsystem initialized.")
    print "Sending log messages to %s" % logfile
    
    return None




from django.utils.translation import ugettext_lazy as _
from django.core.validators import ValidationError

class MutuallyExclusiveWithField(object):
    def __init__(self, other_field_name, error_message=_("Please enter one and only one of those fields.")):
        self.other, self.error_message = other_field_name, error_message
        self.always_test = True

    def __call__(self, field_data, all_data):
        if all_data.get(self.other, False) and field_data or \
           not all_data.get(self.other, False) and not field_data:
            raise ValidationError, self.error_message

class ValidateIfFieldsSame(object):
    def __init__(self, other_field, error_message=_("These fields can not be the same.")):
        self.other_field = other_field
        self.always_test = True
        self.error_message = error_message
    
    def __call__(self, field_data, all_data):
        if field_data == all_data[self.other_field]:
            raise ValidationError, self.error_message


#!/usr/bin/env python
import os
import sys

from kiwi.dist import setup, listfiles, listpackages

from gazpacho import __version__

templates = []
data_files = [
    ('share/doc/gazpacho', ('AUTHORS', 'ChangeLog', 'CONTRIBUTORS',
                            'COPYING', 'README', 'NEWS')),
    ('share/doc/gazpacho/examples', listfiles('examples', '*')),
    ('$datadir/catalogs', listfiles('catalogs', 'base.xml')),
    ('$datadir/pixmaps', listfiles('pixmaps', '*.png')),
    ('$datadir/pixmaps', listfiles('pixmaps', '*.ico')),
    ('$datadir/resources/base', listfiles('resources', 'base', '*.png')),
    ('$datadir/glade', listfiles('glade', '*.glade')),
    ]

def list_plugins():
    ret = []
    for dir in os.listdir('plugins'):
        dest = '$datadir/plugins/%s' % dir
        files1 = listfiles(os.path.join('plugins', dir), '*.plugin')
        if files1:
            ret.append((dest, files1))
        files2 = listfiles(os.path.join('plugins', dir), '*.py')
        if files2:
            ret.append((dest, files2))
    return ret

data_files.extend(list_plugins())

resources = dict(locale='$prefix/share/locale')
global_resources = dict(
    doc='$prefix/share/doc/gazpacho',
    pixmap='$datadir/pixmaps',
    glade='$datadir/glade',
    resource='$datadir/resources',
    catalog='$datadir/catalogs',
    plugins='$datadir/plugins')

# When building the installer, install some extra stuff.
kwargs = {}
if 'bdist_wininst' in sys.argv:
    import shutil
    src = os.path.join('bin', 'gazpacho')
    shutil.copy2(src, os.path.join('bin', 'launch-gazpacho.py'))
    shutil.copy2(src, os.path.join('bin', 'launch-gazpacho.pyw'))
    scripts = ['bin/launch-gazpacho.py',
               'bin/launch-gazpacho.pyw']
elif 'py2exe' in sys.argv:
    import py2exe
    scripts = []
    kwargs['windows'] = [dict(script='bin/gazpacho',
                              icon_resources=[(1, 'gazpacho.ico')]
                              )]
    kwargs['options'] = dict(py2exe=dict(
        packages='encodings',
        includes=('kiwi,cairo,pango,pangocairo,atk,gobject,win32com')))
    import shutil
    if os.path.exists('dist'):
        shutil.rmtree('dist')
    os.mkdir('dist')
    shutil.copytree('glade', 'dist/glade')
    shutil.copytree('pixmaps', 'dist/pixmaps')
    shutil.copytree('catalogs', 'dist/catalogs')
    shutil.copytree('resources', 'dist/resources')
else:
    scripts = ['bin/gazpacho']
    templates.append(('share/applications',
                      ['gazpacho.desktop']))

setup(name='gazpacho',
      version=__version__,
      description='GTK+ GUI Designer',
      author='SICEm S.L.',
      author_email='lgs@sicem.biz',
      url='http://gazpacho.sicem.biz',
      license='LGPL',
      packages=listpackages('gazpacho'),
      scripts=scripts,
      data_files=data_files,
      resources=resources,
      global_resources=global_resources,
      templates=templates,
      **kwargs)

if 'bdist_wininst' in sys.argv:
    for script in scripts:
        os.unlink(script)



from livesettings import *
from django.utils.translation import ugettext_lazy as _

# this is so that the translation utility will pick up the string
gettext = lambda s: s

PAYMENT_GROUP = ConfigurationGroup('PAYMENT_GOOGLE', 
    _('Google Checkout Module Settings'), 
    ordering = 101)

config_register_list(

    StringValue(PAYMENT_GROUP,
        'CART_XML_TEMPLATE',
        description=_("XML Template"),
        help_text=_("The XML template to use when submitting to Google. Probably you should not change this."),
        default = "shop/checkout/google/cart.xml"),
    
    StringValue(PAYMENT_GROUP,
        'CURRENCY_CODE',
        description=_('Currency Code'),
        help_text=_('Currency code for Google Checkout transactions.'),
        default = 'USD'),
    
    StringValue(PAYMENT_GROUP,
        'CHECKOUT_BUTTON_URL',
        description=_('Checkout Button URL'),
        default="http://checkout.google.com/buttons/checkout.gif"),
        
    StringValue(PAYMENT_GROUP,
        'CHECKOUT_BUTTON_SIZE',
        description=_('Checkout Button Size'),
        choices=(
            ('SMALL', _('Small')), 
            ('MEDIUM', _('Medium')), 
            ('LARGE', _('Large'))),
        default='MEDIUM'),
    
    StringValue(PAYMENT_GROUP,
        'POST_URL',
        description=_('Post URL'),
        help_text=_('The Google URL for real transaction posting.'),
        default="https://checkout.google.com/api/checkout/v2/checkout/Merchant/%(MERCHANT_ID)s"),

    StringValue(PAYMENT_GROUP,
        'POST_TEST_URL',
        description=_('Post URL'),
        help_text=_('The Google URL for test transaction posting.'),
        default="https://sandbox.google.com/checkout/api/checkout/v2/checkout/Merchant/%(MERCHANT_ID)s"),

    StringValue(PAYMENT_GROUP,
        'MERCHANT_ID',
        description=_('Merchant ID'),
        default=""),

    StringValue(PAYMENT_GROUP,
        'MERCHANT_KEY',
        description=_('Merchant Key'),
        default=""),

    StringValue(PAYMENT_GROUP,
        'MERCHANT_TEST_ID',
        description=_('Merchant Test ID'),
        default=""),

    StringValue(PAYMENT_GROUP,
        'MERCHANT_TEST_KEY',
        description=_('Merchant Test Key'),
        default=""),
    
    BooleanValue(PAYMENT_GROUP, 
        'SSL', 
        description=_("Use SSL for the module checkout pages?"), 
        default=False),
        
    BooleanValue(PAYMENT_GROUP, 
        'LIVE', 
        description=_("Accept real payments"),
        help_text=_("False if you want to be in test mode"),
        default=False),
        
    ModuleValue(PAYMENT_GROUP,
        'MODULE',
        description=_('Implementation module'),
        hidden=True,
        default = 'payment.modules.google'),
        
    StringValue(PAYMENT_GROUP,
        'KEY',
        description=_("Module key"),
        hidden=True,
        default = 'GOOGLE'),

    StringValue(PAYMENT_GROUP,
        'LABEL',
        description=_('English name for this group on the checkout screens'),
        default = 'Google Checkout',
        help_text = _('This will be passed to the translation utility')),

    StringValue(PAYMENT_GROUP,
        'URL_BASE',
        description=_('The url base used for constructing urlpatterns which will use this module'),
        default = '^google/'),
        
    BooleanValue(PAYMENT_GROUP,
        'EXTRA_LOGGING',
        description=_("Verbose logs"),
        help_text=_("Add extensive logs during post."),
        default=False)

)

PAYMENT_GROUP['TEMPLATE_OVERRIDES'] = {
    'shop/checkout/confirm.html' : 'shop/checkout/google/confirm.html',
}
